package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.service.AttachmentService;
import com.bocloud.cos.utils.FileUtil;
import com.megatron.common.model.GeneralResult;
import io.jsonwebtoken.lang.Assert;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

@Slf4j
@RestController
@RequestMapping("/v1/attachment")
@Tag(name = "附件管理")
public class AttachmentController {

    @Value("${spring.upload.path:/home/cmp}")
    private String path;

    @Autowired
    private AttachmentService attachmentService;
    @Operation(summary = "附件下载")
    @GetMapping("/download")
    public GeneralResult<Void> download(@RequestParam(value = "attachmentId") Long attachmentId,
                                        HttpServletRequest request, HttpServletResponse response) {
        try {
            // 获取合同
            Attachment attachment = attachmentService.get(attachmentId);
            Assert.isTrue(null != attachment && null != attachment.getPath(), "该合同找不到对应附件，无法导出");

            String charsetCode = String.valueOf(StandardCharsets.UTF_8);
            File file = new File(path + "/" + attachment.getPath());
            //中文乱码解决
            String type = request.getHeader("User-Agent").toLowerCase();
            String fileName = attachment.getName();
            // 字符编码格式
            if (type.indexOf("firefox") > 0 || type.indexOf("chrome") > 0) {
                //谷歌或火狐
                fileName = new String(fileName.getBytes(charsetCode), "iso8859-1");
            } else {
                //IE
                fileName = URLEncoder.encode(fileName, charsetCode);
            }
            // 设置响应的头部信息
            response.setHeader("content-disposition", "attachment;filename=" + fileName);
            // 设置响应内容的类型
            response.setContentType(FileUtil.fileContentType(fileName) + "; charset=" + charsetCode);
            // 设置响应内容的长度
            response.setContentLength((int) file.length());
            // 输出
            FileUtil.outStream(Files.newInputStream(file.toPath()), response.getOutputStream());
        } catch (Exception e) {
            log.error("文件下载异常{}", e);
        }
        return new GeneralResult<>(true, "");
    }
}
