package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudBill;
import com.bocloud.cos.intf.CloudBillService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/v1/cloud/bills")
@Tag(name = "账单管理")
public class CloudBillController {

    @Autowired
    private CloudBillService cloudBillService;

    /**
     * 查询账单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(summary = "账单列表查询")
    @GetMapping
    public GeneralResult<GridBean<CloudBill>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudBillService.list(pager, context);
    }

    @Operation(summary = "同步账单数据")
    @GetMapping("/sync")
    public GeneralResult synchronizeData(@RequestParam(value = Common.ID, required = false) Long id,
                                         @RequestParam(value = "month", required = false) String month,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        String result = null;
        try {
            result = cloudBillService.synchronizeData(id, month);
            return new GeneralResult<>(true, result);
        } catch (Exception e) {
            if (null == result) {
                result = e.getMessage();
            }
            return new GeneralResult<>(false, result);
        }
    }

    @Operation(summary = "账单列表导入")
    @PostMapping("/import")
    public GeneralResult importBill(@RequestParam("file") MultipartFile file, @RequestBody CloudBill cloudBill,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudBillService.importBill(file, cloudBill, context);
    }

    /**
     * 导出
     *
     * @param context
     * @param response
     * @param request
     * @return
     */
    @Operation(summary = "导出账单列表")
    @GetMapping("/export")
    public Result export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context,
                         HttpServletResponse response, HttpServletRequest request) {
        GeneralResult GeneralResult = cloudBillService.download(pager, context);
        if (GeneralResult.isSuccess()) {
            String content;
            try {
                @SuppressWarnings("unchecked")
                Map<String, Object> result = JSONObject.parseObject(JSONObject.toJSONString(GeneralResult.getData()),
                        HashMap.class);
                String fileName = result.get("fileName").toString();
                if (fileName.contains("pdf")) {
                    response.setContentType("application/pdf");
                } else {
                    response.setContentType("application/vnd.ms-excel");
                }
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String) result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new GeneralResult(false, "内部错误");
            }
        }
        return GeneralResult;
    }

    /**
     * 获取所有账单资源
     *
     * @return
     */
    @Operation(summary = "获取所有账单资源")
    @GetMapping("/products")
    public Result products(@RequestParam(value = "vendorType", required = false) String vendorType) {
        return cloudBillService.queryProducts(vendorType);
    }

}
