package com.bocloud.cos.booter.controller;

import com.bocloud.cos.intf.ExternalService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 对外展示大屏控制器
 *
 * @author tyl
 * @since 2020/12/23 15:10
 */
@RestController
@RequestMapping("/v1/external")
@Tag(name = "云资源大屏")
public class ExternalScreenController {
    private final ExternalService externalService;

    public ExternalScreenController(ExternalService externalService) {
        this.externalService = externalService;
    }

    @Operation(summary = "资源总览")
    @GetMapping("/resource/overview")
    public GeneralResult resource(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.resourceOverview(dcId, context);
    }

    @Operation(summary = "云主机申请趋势")
    @GetMapping("/vms/trend")
    public GeneralResult vmsTrend(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.vmsTrend(dcId, context);
    }

    @Operation(summary = "云资源申请趋势")
    @GetMapping("/resource/trend")
    public GeneralResult resourceTrend(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.resourceTrend(dcId, context);
    }

    @Operation(summary = "租户云资源统计")
    @GetMapping("/tenants/resource")
    public GeneralResult resourceCount(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.tenantResourceCount(dcId, context);
    }

    @Operation(summary = "项目云资源统计")
    @GetMapping("/projects/resource")
    public GeneralResult storageCount(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.projectResourceCount(dcId, context);
    }

    @Operation(summary = "业务云资源统计")
    @GetMapping("/business/resource")
    public GeneralResult systemResource(@RequestParam(value = "dcId", required = false) Long dcId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return externalService.businessResourceCount(dcId, context);
    }

}
