package com.bocloud.cos.booter.controller;


import com.bocloud.cos.entity.Coupon;
import com.bocloud.cos.entity.Promotion;
import com.bocloud.cos.intf.PromotionService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/promotions")
@Tag(name = "优惠管理")
public class PromotionController {

    @Autowired
    private PromotionService promotionService;

    /**
     * 获取优惠列表
     *
     * @param pager
     * @param condition
     * @param context
     * @return
     */
    @Operation(summary = "获取活动列表")
    @GetMapping
    public GeneralResult list(Pager pager, @RequestParam(value = Common.CONDITION, required = false) String condition,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return promotionService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(), context);
    }

    /**
     * 添加活动
     *
     * @param promotion
     * @param context
     * @return
     */
    @Operation(summary = "添加活动")
    @PostMapping
    public GeneralResult create(@RequestBody Promotion promotion, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != promotion) {
            return promotionService.create(promotion, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 分配优惠券
     *
     * @param coupon
     * @param context
     * @return
     */
    @Operation(summary = "分配优惠券")
    @PostMapping("/{id}/coupons")
    public GeneralResult allocation(@PathVariable(value = Common.ID) Long id,
                                    @RequestBody Coupon coupon, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != coupon) {
            return promotionService.allocation(id, coupon.getTenantId(), coupon.getAllocateCount(), context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改活动
     *
     * @param id
     * @param promotion
     * @param context
     * @return
     */
    @Operation(summary = "修改活动")
    @PutMapping("/{id}")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id, @RequestBody Promotion promotion,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != promotion) {
            promotion.setId(id);
            return promotionService.modify(promotion, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 删除活动
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "删除活动")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return promotionService.remove(id, context);
    }

    /**
     * 查看活动详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "查看活动详情")
    @GetMapping("/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return promotionService.detail(id, context);
    }

    /**
     * 查询活动下租户的分配情况
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询活动下租户的分配情况")
    @GetMapping("/{id}/tenants")
    public GeneralResult promotionTenants(@PathVariable(value = Common.ID) Long id, Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return promotionService.promotionTenants(pager, id, context);
    }


    /**
     * 查询活动下具体优惠券
     *
     * @param pager
     * @return
     */
    @Operation(summary = "查询活动下具体优惠券")
    @GetMapping("/coupons")
    public GeneralResult coupons(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return promotionService.coupon(pager, context);
    }


}
