package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.Recycled;
import com.bocloud.cos.intf.RecycledService;
import com.bocloud.cos.model.ResourceRecycledParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 回收站
 */
@Tag(name = "回收站")
@RestController
@RequestMapping("/v1/recycles")
public class RecycledController {
    @Autowired
    private RecycledService recycledService;

    @Operation(summary = "加入回收站")
    @PostMapping
    public GeneralResult create(@RequestParam(value = Common.PARAMS) String params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            Recycled recycled = JSONObject.parseObject(object.toJSONString(), Recycled.class);
            return recycledService.create(recycled, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    @Operation(summary = "查询回收站列表")
    @GetMapping
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(), context);
    }

    @Operation(summary = "删除回收站记录")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.remove(id, context);
    }

    @Operation(summary = "根据订单删除回收站记录")
    @DeleteMapping("/sn")
    public GeneralResult removeBySn(@RequestParam String sn, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (!StringUtils.isEmpty(sn)) {
            return recycledService.remove(sn, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    @Operation(summary = "回收站还原")
    @PatchMapping("/{id}/restore")
    public GeneralResult restore(@PathVariable(value = Common.ID) Long id,
                                 @RequestBody ResourceRecycledParam resourceRecycledParam,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(JSON.toJSONString(resourceRecycledParam));
        if (null != object) {
            Long resourceId = object.getLong("resourceId");
            String type = object.getString("type");
            return recycledService.restore(resourceId, id, type, context);
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 清空回收站
     *
     * @param context
     * @return
     */
    @Operation(summary = "删除回收站记录")
    @DeleteMapping()
    @Deprecated
    public GeneralResult clear(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return recycledService.clear(context);
    }

}
