package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.ServiceSla;
import com.bocloud.cos.intf.ServiceSlaService;
import com.bocloud.cos.params.ServiceSlaParams;
import com.bocloud.cos.params.SlaActionParams;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 服务SLA
 *
 * @author weiwei
 * @version 1.0
 * @since 2020年3月11日
 */
@RestController
@RequestMapping("/v1/services/slas")
@Tag(name = "服务sla管理")
public class ServiceSlaController {

    private final ServiceSlaService serviceSlaService;

    public ServiceSlaController(ServiceSlaService serviceSlaService) {
        this.serviceSlaService = serviceSlaService;
    }

    /**
     * 获取服务SLA列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(summary = "获取服务sla列表")
    public GeneralResult<GridBean<ServiceSla>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSlaService.list(pager, context);
    }

    /**
     * 添加服务SLA
     *
     * @param serviceSlaParams
     * @param context
     * @return
     */
    @PostMapping
    @Operation(summary = "添加服务sla")
    public Result create(@RequestBody ServiceSlaParams serviceSlaParams,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSlaService.create(serviceSlaParams, context);
    }

    /**
     * 修改服务SLA
     *
     * @param id
     * @param slaParams
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改服务sla")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody ServiceSlaParams slaParams, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSlaService.modify(id, slaParams, context);
    }

    /**
     * 删除服务SLA
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除服务sla")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSlaService.remove(id, context);
    }

    /**
     * 查看服务SLA详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "查看服务sla详情")
    public GeneralResult<ServiceSla> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceSlaService.detail(id);
    }

    /**
     * 操作服务SLA
     *
     * @param id 作业ID
     * @return 操作结果
     */
    @PatchMapping(value = "/{id}")
    @Operation(summary = "操作服务sla")
    public Result action(@PathVariable(value = Common.ID) Long id,
                         @RequestBody SlaActionParams action,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (action.getAction()) {
            case enable:
            case disable:
                return serviceSlaService.updateStatus(id, action.getAction().name(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类服务SLA操作");
        }
    }


    /**
     * 批量操作服务SLA
     *
     * @param action sla批量操作对象
     * @return 操作结果
     * @see SlaActionParams
     */
    @PatchMapping
    @Operation(summary = "操作服务sla")
    public Result batchAction(@RequestBody SlaActionParams action,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (action.getAction()) {
            case enable:
            case disable:
                return serviceSlaService.batchUpdateStatus(action.getIds(), action.getAction().name(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类服务SLA操作");
        }
    }
}
