package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.ShoppingCart;
import com.bocloud.cos.intf.ShoppingCartService;
import com.bocloud.cos.model.ApplicationParam;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 购物车控制器
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/2/18
 */
@RestController
@RequestMapping("/v1/shopping/cart")
@Tag(name = "购物车管理")
public class ShoppingCartController {

    private final ShoppingCartService shoppingCartService;

    public ShoppingCartController(ShoppingCartService shoppingCartService) {
        this.shoppingCartService = shoppingCartService;
    }

    /**
     * 查询购物车清单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(summary = "查询购物车清单列表")
    @GetMapping
    public GeneralResult<GridBean<ShoppingCart>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.list(pager, context);
    }

    /**
     * 删除购物车清单
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "删除购物车清单")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.remove(id, context);
    }

    /**
     * 查询购物车详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询购物车详情")
    @GetMapping("/{id}")
    public GeneralResult<ShoppingCart> detail(@PathVariable(value = Common.ID) Long id) {
        return shoppingCartService.detail(id);
    }

    /**
     * 清空购物车
     *
     * @param context
     * @return
     */
    @Operation(summary = "清空购物车")
    @PutMapping
    public Result empty(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.empty(context);
    }


    /**
     * 更新购物车
     *
     * @param id
     * @return
     */
    @Operation(summary = "更新购物车")
    @PutMapping("/{id}")
    public Result empty(@PathVariable Long id, @RequestBody ApplicationParam applicationParam, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.update(id, applicationParam, context);
    }


    /**
     * 添加到购物车
     *
     * @param applicationParam
     * @param context
     * @return
     */
    @Operation(summary = "添加到购物车")
    @PostMapping
    public Result addToShoppingCar(@RequestBody ApplicationParam applicationParam, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.create(applicationParam, context);
    }

    /**
     * 批量申请
     *
     * @param context
     * @return
     */
    @Operation(summary = "购物车批量申请")
    @PostMapping("/apply")
    public Result apply(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return shoppingCartService.batchApply(context);
    }
}
