package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessCollectionMetric;
import com.bocloud.cos.entity.assess.AssessCollectionObject;
import com.bocloud.cos.service.assess.AssessCollectionMetricService;
import com.bocloud.cos.service.assess.AssessCollectionObjectService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 考核采集配置
 *
 * @author lym
 */
@RestController
@RequestMapping("/v1/assess/collection")
@Tag(name = "考核采集配置管理")
public class AssessCollectionController {

    @Autowired
    private AssessCollectionObjectService collectionObjectService;

    @Autowired
    private AssessCollectionMetricService collectionMetricService;


    @Operation(summary = "获取考核采集对象列表")
    @GetMapping("/object")
    public GeneralResult<GridBean<AssessCollectionObject>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessCollectionObject> result = collectionObjectService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "新增考核采集对象")
    @PostMapping("/object")
    public GeneralResult<?> save(@RequestBody AssessCollectionObject collectionObject, @Value(Common.REQ_CONTEXT) RequestContext context) {
        collectionObjectService.save(collectionObject, context);
        return new GeneralResult<>(true, "新增考核采集对象成功");
    }

    @Operation(summary = "编辑考核采集对象")
    @PatchMapping("/object")
    public GeneralResult<?> edit(@RequestBody AssessCollectionObject collectionObject, @Value(Common.REQ_CONTEXT) RequestContext context) {
        collectionObjectService.update(collectionObject, context);
        return new GeneralResult<>(true, "编辑成功");
    }

    @Operation(summary = "移除考核采集对象")
    @DeleteMapping("/object/{id}")
    public GeneralResult<?> remove(@PathVariable(value = Common.ID) Long id) {
        collectionObjectService.remove(id);
        return new GeneralResult<>(true, "删除成功");
    }

    @Operation(summary = "获取考核采集指标列表")
    @GetMapping("/metric")
    public GeneralResult<GridBean<AssessCollectionMetric>> listMetric(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessCollectionMetric> result = collectionMetricService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "新增考核采集指标")
    @PostMapping("/metric")
    public GeneralResult<?> save(@RequestBody AssessCollectionMetric collectionMetric, @Value(Common.REQ_CONTEXT) RequestContext context) {
        collectionMetricService.save(collectionMetric, context);
        return new GeneralResult<>(true, "新增考核采集指标成功");
    }

    @Operation(summary = "编辑考核指标")
    @PatchMapping("/metric")
    public GeneralResult<?> edit(@RequestBody AssessCollectionMetric collectionMetric, @Value(Common.REQ_CONTEXT) RequestContext context) {
        collectionMetricService.update(collectionMetric, context);
        return new GeneralResult<>(true, "编辑成功");
    }

    @Operation(summary = "移除采集指标")
    @DeleteMapping("/metric/{id}")
    public GeneralResult<?> removeMetric(@PathVariable(value = Common.ID) Long id) {
        collectionMetricService.remove(id);
        return new GeneralResult<>(true, "删除成功");
    }
}
