package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessConformityStandard;
import com.bocloud.cos.service.assess.AssessConformityStandardService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author wpj
 * @since 2023/10/27
 */


@RestController
@RequestMapping("/v1/assess/conformity/standard")
@Tag(name = "合格标准管理")
public class AssessConformityStandardController {

    @Autowired
    private AssessConformityStandardService assessConformityStandardService;

    @Operation(summary = "新增合格标准管理")
    @PostMapping
    public GeneralResult<?> save(@RequestBody AssessConformityStandard assessConformityStandard, @Value(Common.REQ_CONTEXT) RequestContext context) {
            return assessConformityStandardService.save(assessConformityStandard, context);
    }


    @Operation(summary = "编辑合格标准")
    @PatchMapping
    public GeneralResult<?> editConformityStandard(@RequestBody AssessConformityStandard assessConformityStandard, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return assessConformityStandardService.update(assessConformityStandard, context);
    }

    @Operation(summary = "删除合格标准")
    @DeleteMapping("/{id}")
    public GeneralResult<?> removeConformityStandard(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessConformityStandardService.delete(id, context);
        return new GeneralResult<>(true, "移除合格标准成功");
    }

    @Operation(summary = "获取合格标准列表")
    @GetMapping("/list")
    public GeneralResult<GridBean<AssessConformityStandard>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessConformityStandard> result = assessConformityStandardService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }


    @Operation(summary = "获取合格标准详情")
    @GetMapping("/query")
    public AssessConformityStandard query(@RequestParam(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return assessConformityStandardService.getConformityStandard(id);
    }

}