package com.bocloud.cos.booter.controller.assess;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.cos.entity.assess.AssessLevel;
import com.bocloud.cos.service.assess.AssessLevelService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * 评价等级管理
 *
 * @author lym
 */
@RestController
@RequestMapping("/v1/assess/levels")
@Tag(name = "评价等级管理")
public class AssessLevelController {

	@Autowired
	private AssessLevelService assessLevelService;

	@GetMapping
	@Operation(summary = "获取评价等级列表")
	public GeneralResult<GridBean<AssessLevel>> listIndex(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		GridBean<AssessLevel> result = assessLevelService.list(pager, context);
		return new GeneralResult<>(true, result, "查询成功");
	}

	@GetMapping("/{id}")
	@Operation(summary = "获取评价等级")
	public GeneralResult<?> query(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		AssessLevel level = assessLevelService.query(id);
		return new GeneralResult<>(true, level, "获取评价等级成功");
	}

	@PostMapping
	@Operation(summary = "新增评价等级")
	public GeneralResult<?> save(@RequestBody AssessLevel level, @Value(Common.REQ_CONTEXT) RequestContext context) {
		assessLevelService.save(level, context);
		return new GeneralResult<>(true, "新增评价等级成功");
	}

	@PutMapping(value = "/{id}")
	@Operation(summary = "编辑评价等级")
	public GeneralResult<?> edit(@RequestBody AssessLevel level, @Value(Common.REQ_CONTEXT) RequestContext context) {
		assessLevelService.update(level, context);
		return new GeneralResult<>(true, "编辑评价等级成功");
	}

	@DeleteMapping("/{id}")
	@Operation(summary = "移除评价等级")
	public GeneralResult<?> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		assessLevelService.remove(id, context);
		return new GeneralResult<>(true, "移除评价等级成功");
	}

}
