package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessMetric;
import com.bocloud.cos.entity.assess.AssessMetricClassify;
import com.bocloud.cos.entity.assess.AssessRule;
import com.bocloud.cos.service.assess.AssessMetricClassifyService;
import com.bocloud.cos.service.assess.AssessMetricService;
import com.bocloud.cos.service.assess.AssessRuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 指标池管理
 *
 * @author lym
 */
@RestController
@RequestMapping("/v1/assess")
@Tag(name = "指标池管理")
public class AssessMetricController {

    @Autowired
    private AssessMetricClassifyService assessMetricClassifyService;
    @Autowired
    private AssessMetricService assessMetricService;
    @Autowired
    private AssessRuleService assessRuleService;

    @Operation(summary = "新增考核指标分类")
    @PostMapping("/category")
    public GeneralResult<?> saveCategory(@RequestBody AssessMetricClassify metricClassify, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricClassifyService.save(metricClassify, context);
        return new GeneralResult<>(true, "新增考核指标分类成功");
    }

    @Operation(summary = "编辑考核指标分类")
    @PatchMapping("/category")
    public GeneralResult<?> editCategory(@RequestBody AssessMetricClassify metricClassify, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricClassifyService.update(metricClassify, context);
        return new GeneralResult<>(true, "编辑考核指标分类成功");
    }

    @Operation(summary = "移除考核指标分类")
    @DeleteMapping("/category/{id}")
    public GeneralResult<?> removeCategory(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricClassifyService.remove(id, context);
        return new GeneralResult<>(true, "移除考核指标分类成功");
    }

    @Operation(summary = "获取考核指标分类")
    @GetMapping("/category")
    public GeneralResult<?> queryCategory(@Value(Common.REQ_CONTEXT) RequestContext context) {
        Object result = assessMetricClassifyService.query();
        return new GeneralResult<>(true, result, "获取考核指标分类成功");
    }

    @Operation(summary = "获取考核指标列表")
    @GetMapping("/metric")
    public GeneralResult<GridBean<AssessMetric>> listMetric(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessMetric> result = assessMetricService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "新增考核指标")
    @PostMapping("/metric")
    public GeneralResult<?> saveMetric(@RequestBody AssessMetric assessMetric, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricService.save(assessMetric, context);
        return new GeneralResult<>(true, "新增考核指标成功");
    }

    @Operation(summary = "编辑考核指标")
    @PatchMapping("/metric")
    public GeneralResult<?> editMetric(@RequestBody AssessMetric assessMetric, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricService.update(assessMetric, context);
        return new GeneralResult<>(true, "编辑考核指标成功");
    }

    @Operation(summary = "移除考核指标")
    @DeleteMapping("/metric/{id}")
    public GeneralResult<?> removeMetric(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessMetricService.remove(id, context);
        return new GeneralResult<>(true, "移除考核分类成功");
    }

    @Operation(summary = "获取没有被绑定到指定某个任务下的指标")
    @GetMapping("/metric/not/bind/task")
    public GeneralResult<List<AssessMetric>> listMetric(@RequestParam(value = "classifyId") Long classifyId,
                                                        @RequestParam(value = "taskId") Long taskId) {
        List<AssessMetric> list = assessMetricService.queryNotBindTask(classifyId, taskId);
        return new GeneralResult<>(true, list, "查询成功");
    }

    @Operation(summary = "获取某个指定任务下绑定的指标")
    @GetMapping("/metric/bind/task")
    public GeneralResult<List<AssessMetric>> listMetric(@RequestParam(value = "taskId") Long taskId) {
        List<AssessMetric> list = assessMetricService.queryByTaskId(taskId);
        return new GeneralResult<>(true, list, "查询成功");
    }

    @Operation(summary = "获取考核规则列表")
    @GetMapping("/rule")
    public GeneralResult<GridBean<AssessRule>> listRule(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessRule> result = assessRuleService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "新增考核规则")
    @PostMapping("/rule")
    public GeneralResult<?> saveRule(@RequestBody AssessRule assessRule, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRuleService.save(assessRule, context);
        return new GeneralResult<>(true, "新增考核规则成功");
    }

    @Operation(summary = "编辑考核规则")
    @PatchMapping("/rule")
    public GeneralResult<?> editRule(@RequestBody AssessRule assessRule, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRuleService.update(assessRule, context);
        return new GeneralResult<>(true, "编辑考核规则成功");
    }

    @Operation(summary = "移除考核规则")
    @DeleteMapping("/rule/{id}")
    public GeneralResult<?> removeRule(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRuleService.remove(id, context);
        return new GeneralResult<>(true, "移除考核规则成功");
    }

}
