package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.service.assess.AssessRecordService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 考核统计管理控制
 *
 * @author lym
 */
@RestController
@RequestMapping("/v1/assess/statistics")
@Tag(name = "考核绩效统计管理")
public class AssessStatisticsController {

    @Autowired
    private AssessRecordService assessRecordService;

    @Operation(summary = "绩效分析-考核指标值")
    @GetMapping("/score/rank")
    public GeneralResult<?> scoreRank(@RequestParam(value = "classifyId") Long classifyId,
                                   @RequestParam(value = "cycleValue") String cycleValue,
                                   @RequestParam(value = "num", required = false, defaultValue = "5") Integer num) {
        Object result = assessRecordService.scoreRank(classifyId, cycleValue, num);
        return new GeneralResult<>(true, result, "云服务商考核查询成功");
    }

    @Operation(summary = "考核分数趋势统计")
    @GetMapping("/score/trend")
    public GeneralResult<?> scoreTrend(@RequestParam(value = "classifyId") Long classifyId,
                                    @RequestParam(value = "year") String year) {
        Object result = assessRecordService.scoreTrend(classifyId, year);
        return new GeneralResult<>(true, result, "考核分数趋势统计查询成功");
    }

    @Operation(summary = "考核分值TOP")
    @GetMapping("/score/top")
    public GeneralResult<?> top(@RequestParam(value = "classifyId") Long classifyId,
                             @RequestParam(value = "cycleValue") String cycleValue,
                             @RequestParam(value = "sort", required = false, defaultValue = "desc") String sort,
                             @RequestParam(value = "limit", required = false, defaultValue = "5") Integer limit) {
        Object result = assessRecordService.scoreRange(classifyId, cycleValue, sort, limit);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "考核分值占比")
    @GetMapping("/score/proportion")
    public GeneralResult<?> proportion(@RequestParam(value = "classifyId") Long classifyId,
                                    @RequestParam(value = "cycleValue") String cycleValue) {
        Object result = assessRecordService.scoreProportion(classifyId, cycleValue);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "考核结果查询")
    @GetMapping("/score/result")
    public GeneralResult<?> scoreResult(Pager pager, @RequestParam(value = "classifyId") Long classifyId,
                                     @RequestParam(value = "cycleValue") String cycleValue) {
        Object result = assessRecordService.scoreResult(pager, classifyId, cycleValue);
        return new GeneralResult<>(true, result, "考核结果查询成功");
    }

    @Operation(summary = "考核结果详情")
    @GetMapping("/score/result/detail")
    public GeneralResult<?> scoreResultDetail(@RequestParam(value = "recordId") Long recordId) {
        Object result = assessRecordService.scoreResultDetail(recordId);
        return new GeneralResult<>(true, result, "考核结果详情查询成功");
    }
}
