package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessObject;
import com.bocloud.cos.entity.assess.AssessTask;
import com.bocloud.cos.entity.assess.AssessTaskMetric;
import com.bocloud.cos.service.assess.AssessTaskMetricService;
import com.bocloud.cos.service.assess.AssessTaskService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 考核任务访问控制
 *
 * @author lym
 */
@RestController
@RequestMapping("/v1/assess/task")
@Tag(name = "考核任务管理")
public class AssessTaskController {

    @Autowired
    private AssessTaskService taskService;

    @Autowired
    private AssessTaskMetricService taskMetricService;

    @Operation(summary = "新增考核任务")
    @PostMapping
    public GeneralResult<?> save(@RequestBody AssessTask task, @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskService.save(task, context);
        return new GeneralResult<>(true, "新增考核任务成功");
    }

    @Operation(summary = "编辑考核任务")
    @PatchMapping
    public GeneralResult<?> editAssessTask(@RequestBody AssessTask task, @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskService.update(task, context);
        return new GeneralResult<>(true, "编辑考核任务成功");
    }

    @Operation(summary = "获取考核任务详情")
    @GetMapping("/{id}")
    public GeneralResult<?> getAssessTask(@PathVariable(value = Common.ID) Long taskId) {
        return new GeneralResult<>(true, taskService.detail(taskId), "获取考核任务详情成功");
    }

    @Operation(summary = "移除考核任务")
    @DeleteMapping("/{id}")
    public GeneralResult<?> removeAssessTask(@PathVariable(value = Common.ID) Long id) {
        taskService.remove(id);
        return new GeneralResult<>(true, "移除考核任务成功");
    }

    @Operation(summary = "获取考核任务列表")
    @GetMapping("/list")
    public GeneralResult<GridBean<AssessTask>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessTask> result = taskService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "获取考核对象列表")
    @GetMapping("/object/list/{id}")
    public GeneralResult<GridBean<AssessObject>> listAssessObject(Pager pager, @PathVariable(value = Common.ID) Long taskId) {
        GridBean<AssessObject> result = taskService.listAssessObject(pager, taskId);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "任务绑定关联指标")
    @PatchMapping("/bind/metric")
    public GeneralResult<?> taskBindMetric(@RequestParam(value = "taskId") Long taskId,
                                           @RequestParam(value = "metricIds") List<Long> metricIds,
                                           @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskMetricService.taskBindMetric(taskId, metricIds, context);
        return new GeneralResult<>(true, "添加成功");
    }

    @Operation(summary = "保存任务指标信息")
    @PostMapping("/update/metric")
    public GeneralResult<?> taskBindMetric(@RequestBody List<AssessTaskMetric> assessTaskMetrics,
                                           @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskMetricService.updateAssessTaskMetric(assessTaskMetrics, context);
        return new GeneralResult<>(true, "添加成功");
    }

    @Operation(summary = "获取考核任务绑定的指标明细")
    @GetMapping("/metric/list")
    public GeneralResult<List<AssessTaskMetric>> queryTaskMetric(@RequestParam(value = "taskId") Long taskId,
                                                                 @RequestParam(value = "metricName", required = false) String metricName) {
        List<AssessTaskMetric> assessTaskMetrics = taskMetricService.listDetailByTaskId(taskId, metricName);
        return new GeneralResult<>(true, assessTaskMetrics, "查询成功");
    }

    @Operation(summary = "任务发布")
    @PutMapping("/publish/{id}")
    public GeneralResult<?> taskPublish(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskService.taskPublish(id, context);
        return new GeneralResult<>(true, "任务发布成功");
    }

    @Operation(summary = "转为模板")
    @PatchMapping("/convert/template/{id}")
    public GeneralResult<?> convertToTemplate(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        taskService.convertToTemplate(id, context);
        return new GeneralResult<>(true, "操作成功");
    }
}
