package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.cache.RequestMappingCache;
import com.megatron.framework.core.CurrentService;
import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.integration.listener.PluginListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * ApiInfo 监听事件
 *
 * @author zhangyf
 */
@Slf4j
@Component
public class PluginApiListener implements PluginListener {


    private final CurrentService currentService;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final StringRedisTemplate redisTemplate;
    private final RequestMappingCache requestMappingCache;

    public PluginApiListener(CurrentService currentService, ThreadPoolTaskExecutor taskExecutor,
                             StringRedisTemplate redisTemplate, RequestMappingCache requestMappingCache) {
        this.currentService = currentService;
        this.taskExecutor = taskExecutor;
        this.redisTemplate = redisTemplate;
        this.requestMappingCache = requestMappingCache;
    }

    @Override
    public void startSuccess(PluginInfo pluginInfo) {
        taskExecutor.execute(() -> {
            List<RequestMappingCache.ApiInfo> apiInfos = requestMappingCache.getApiInfoList();
            redisTemplate.opsForHash().put("api_list", currentService.getService().getName(), JSONObject.toJSONString(apiInfos));
            log.info("插件[{}]添加Api信息到缓存成功", pluginInfo.getPluginId());
        });
    }
}
