package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.intf.ResourceApplicationService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

@RequiredArgsConstructor
@Component
@Slf4j
public class RosChargingListener {

    private final ResourceApplicationService resourceApplicationService;

    /**
     * 资源编排结果
     *
     * @param message 回调消息
     *                {
     *                "success":"true"|"false",
     *                "orderId": 1L,
     *                "message":"提示消息"
     *                }
     */
    @RabbitListener(queues = "ros.charging.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (StringUtils.isBlank(content)) {
                return;
            }
            log.info("receive ros.charging.queue message：{}", content);
            JSONObject object = JSONObject.parseObject(content, JSONObject.class);
            if (null == object || object.isEmpty()) {
                return;
            }
            boolean success = object.getBoolean("success");
            long orderId = object.getLongValue("orderId");
            String msg = object.getString("message");
            resourceApplicationService.rosCallBack(success, orderId, msg);
        } catch (RuntimeException e) {
            log.error("ros callback error: {}", e.getMessage(), e);
        }
    }
}
