package com.bocloud.cos.entity;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 合同实体类
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Table("contract")
public class Contract extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id;
    @Column("tenant_id")
    private Long tenantId;
    @Column("apply_sn")
    private String applySn;
    @Column("company_name")
    private String companyName;
    @Column("company_address")
    private String companyAddress;
    @Column("contact_name")
    private String contactName;
    @Column("contact_phone")
    private String contactPhone;

    @Column("start_time")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date startTime;

    @Column("end_time")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date endTime;

    // 0 已提交 1 已完成 2 已取消
    @Column("status")
    private Integer status;
    @Column("total_money")
    private BigDecimal totalMoney;
    @IgnoreUpdate
    @Column(value = "gmt_create",nullable = false)
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;
    @IgnoreUpdate
    @Column("creator_id")
    private Long creatorId;
    @Column("mender_id")
    private Long menderId;
    @Column("attachment_id")
    private Long attachmentId;
    @Column("is_deleted")
    private Boolean deleted;
    @Column("remark")
    private String remark;
    @IgnoreAll
    private String tenantAccount;
    @IgnoreAll
    private List<ServiceOrder> orders;
    @IgnoreAll
    private List<Long> billIds;
    @IgnoreAll
    private String attachmentPath;


}


