package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 合格标准实体类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_conformity_standard")
public class AssessConformityStandard extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 合格标准名称
     */
    @Column("name")
    private String name;
    /**
     * 所属分类ID
     */
    @Column("classify_id")
    private Long classifyId;
    /**
     * 标准范围（绩效总分、单个指标）
     */
    @Column("standard_range")
    private StandardRangeEnum standardRange;
    /**
     * 标准区间设置
     */
    @Column("range_conf")
    private String rangeConf;
    /**
     * 评价等级ID
     */
    @Column("level_id")
    private Long levelId;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 备注
     */
    @Column("remark")
    private String remark;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    @IgnoreAll
    private String classifyName;

    @IgnoreAll
    private String standardRangeName;

    @IgnoreAll
    private String creator;

    @Getter
    public enum StandardRangeEnum {
        // 标准范围
        TOTAL("绩效总分值", "TOTAL"),
        SINGLE("合格指标数", "SINGLE");

        final String name;
        final String code;

        StandardRangeEnum(String name, String code) {
            this.name = name;
            this.code = code;
        }
    }
    @Data
    public static class StandardInterval {
        /**
         * 绩效等级
         */
        String name;
        /**
         * 操作符
         */
        String operator;
        /**
         * 值
         */
        Integer value;
        /**
         * 操作符
         */
        String operator2;
        /**
         * 值
         */
        Integer value2;
        /**
         * 分值
         */
        Integer score;
        /**
         * 级别
         */
        Integer level;
        /**
         * 是否合格
         */
        Boolean qualified;
    }
}
