package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 考核指标实体类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_metric")
public class AssessMetric extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 考核指标名称
     */
    @Column("name")
    private String name;
    /**
     * 所属分类ID
     */
    @Column("classify_id")
    private Long classifyId;
    /**
     * 所属考核分类ID
     */
    @Column("metric_classify_id")
    private Long metricClassifyId;
    /**
     * 考核指标唯一标识
     */
    @Column("metric_unique_id")
    private String metricUniqueId;
    /**
     * 采集方式（AUTO: 自动；MANUAL：手动）
     */
    @Column("acquisition_mode")
    private String acquisitionMode;
    /**
     * 采集对象ID
     */
    @Column("collection_obj_id")
    private Long collectionObjId;
    /**
     * 采集指标ID
     */
    @Column("collection_metric_id")
    private Long collectionMetricId;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    /**
     * 考核分类名称
     */
    @IgnoreAll
    private String categoryName;
    /**
     * 考核规则数量
     */
    @IgnoreAll
    private Long ruleCount;

    @IgnoreAll
    private List<AssessRule> assessRules;

    @IgnoreAll
    private boolean isBuildMetric;
}
