package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 考核记录实体类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_record")
public class AssessRecord extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 任务ID
     */
    @Column("task_id")
    private Long taskId;
    /**
     * 考核对象ID
     */
    @Column("assess_obj_id")
    private Long assessObjId;
    /**
     * 状态
     */
    @Column("status")
    private String status;
    /**
     * 是否执行
     */
    @Column("is_execute")
    private Boolean execute;
    /**
     * 执行人ID
     */
    @Column("executor_id")
    private Long executorId;
    /**
     * 执行结果
     */
    @Column("execute_result")
    private String executeResult;
    /**
     * 执行分数
     */
    @Column("execute_score")
    private Double executeScore;
    /**
     * 合格数量
     */
    @Column("qualified_count")
    private Integer qualifiedCount;
    /**
     * 不合格数量
     */
    @Column("unqualified_count")
    private Integer unqualifiedCount;
    /**
     * 绩效等级
     */
    @Column("performance_level")
    private String performanceLevel;
    /**
     * 执行开始时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_start")
    private Date gmtStart;
    /**
     * 执行结束时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_end")
    private Date gmtEnd;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 备注
     */
    @Column("remark")
    private String remark;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    /**
     * 考核类型
     */
    @IgnoreAll
    private String className;
    /**
     * 考核任务名称
     */
    @IgnoreAll
    private String taskName;
    /**
     * 考核对象名称
     */
    @IgnoreAll
    private String assessObj;
    /**
     * 创建人
     */
    @IgnoreAll
    private String creator;
    /**
     * 执行人
     */
    @IgnoreAll
    private String executor;
    /**
     * 考核周期类型（月度、季度、年度）
     */
    @IgnoreAll
    private AssessTask.CycleTypeEnum cycleType;
    /**
     * 考核周期值
     */
    @IgnoreAll
    private String cycleValue;
    /**
     * 评分标准
     */
    @IgnoreAll
    private String conformityStandard;

}
