package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 考核记录与考核指标细则实体类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_record_metric")
public class AssessRecordMetric extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 考核记录ID
     */
    @Column("record_id")
    private Long recordId;
    /**
     * 指标规则ID
     */
    @Column("metric_rule_id")
    private Long metricRuleId;
    /**
     * 权重
     */
    @Column("weight")
    private Integer weight;
    /**
     * 最高分
     */
    @Column("max_score")
    private Integer maxScore;
    /**
     * 最终评分
     */
    @Column("score")
    private Double score;
    /**
     * 分数等级（用于前端回显选中对应的得分）
     */
    @Column("score_level")
    private Integer scoreLevel;
    /**
     * 是否合格（true：合格；false：不合格）
     */
    @Column("qualified")
    private Boolean qualified;
    /**
     * 上传材料附件ID
     */
    @Column("attachment_id")
    private Long attachmentId;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 备注
     */
    @Column("remark")
    private String remark;

    /**
     * 指标名称
     */
    @IgnoreAll
    private String metricName;
    /**
     * 采集方式（AUTO: 自动；MANUAL：手动）
     */
    @IgnoreAll
    private String acquisitionMode;
    /**
     * 所属指标ID
     */
    @IgnoreAll
    private Long metricId;
    /**
     * 规则内容
     */
    @IgnoreAll
    private String ruleContent;
    /**
     * 提交材料项
     */
    @IgnoreAll
    private String submissionItem;
    /**
     * 分值设定
     */
    @IgnoreAll
    private String scoreSetting;
    /**
     * 数据函数
     */
    @IgnoreAll
    private String dataFunction;
    /**
     * 评分方式（数据区间、提交材料满足项等）
     */
    @IgnoreAll
    private String scoringManner;
    /**
     * 附件上传路径
     */
    @IgnoreAll
    private String uploadPath;
    /**
     * 上传材料名称
     */
    @IgnoreAll
    private String uploadMaterials;
}
