package com.bocloud.cos.repository;

import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.bocloud.cos.entity.CloudBill;
import com.megatron.database.core.intf.impl.BasicGenericDao;

import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 外部账单
 *
 * @author admin
 */
@Repository("cloudBillRepository")
public class CloudBillRepository extends BasicGenericDao<CloudBill, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public CloudBillRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<CloudBill> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.*,c.vendor_name ,r.name region_name  from cloud_bill a  left join cloud_vendor_account c on a.vendor_id = c.id "
                + " left join " + cmpDb + ".region r on a.vendor_id = r.vendor_id and a.region_id = r.region_id " + " where 1 = 1";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(CloudBill.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from cloud_bill a where 1 = 1";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<CloudBill> list() {
        String sql = "select * from cloud_bill";
        return this.list(CloudBill.class, sql, new HashMap<>(8));
    }

    public CloudBill query(Long id) {
        String sql = "select * from cloud_bill where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<CloudBill> list = this.list(CloudBill.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<CloudBill> listByVendor(Long vendorId, String regionId) {
        String sql = "select * from cloud_bill a where a.vendor_id = :vendorId  ";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        if (!StringUtils.isEmpty(regionId)) {
            sql += " and a.region_id = :regionId ";
            params.put(Common.REGIONID, regionId);
        }
        return this.list(CloudBill.class, sql, params);
    }

    public CloudBill listByVendorAndOrderSn(Long vendorId, String orderSn) {
        String sql = new StringBuilder()
                .append("select * from cloud_bill where vendor_id = ")
                .append(vendorId).append(" and order_sn = '").append(orderSn)
                .append("' limit 1").toString();
        List<CloudBill> list = this.list(CloudBill.class, sql, null);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public void deleteByVendor(Long vendorId, String regionId) {
        String sql = "delete from cloud_bill where vendor_id = :vendorId  ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        if (!StringUtils.isEmpty(regionId)) {
            sql += " and region_id = :regionId ";
            params.put(Common.REGIONID, regionId);
        }
        this.execute(sql, params);
    }

    public boolean delete(Long id) {
        String sql = "delete from cloud_bill where id = :id  ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByVid(Long vendorId, String month) {
        String sql = "delete from cloud_bill where vendor_id = :vendorId and month = :month";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("month", month);
        return this.execute(sql, params) > 0;
    }

    public List<CloudBill> queryProducts(String vendorType) {

        StringBuilder sql = new StringBuilder();
        sql.append("select product_name from cloud_bill where 1=1 ");
        if (StringUtils.isNotBlank(vendorType)) {
            sql.append(" and vendor_type = '").append(vendorType).append("'");
        }
        sql.append(" group by product_name");
        return this.list(CloudBill.class, sql.toString(), null);
    }
    public boolean isExistByResourceId(String resourceId, Date gmtCreate) {
        String sql = "select count(1) from cloud_bill where resource_id = :resourceId and gmt_create = :gmtCreate ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("gmtCreate", gmtCreate);
        return this.countQuery(sql, params) > 0;
    }
}
