package com.bocloud.cos.repository;

import com.bocloud.cos.entity.Contract;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;

import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public class ContractRepository extends BasicGenericDao<Contract, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public ContractRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Contract> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select c.*,t.account tenant_account,a.path as attachment_path from contract c " +
                " left join " + cmpDb + ".tenant t on t.id = c.tenant_id " +
                " left join attachment a on a.id = c.attachment_id" +
                " where 1 = 1 and c.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "c");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Contract.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from contract a where 1 = 1 and a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Contract query(Long id) {
        String sql = "select a.*, t.account tenant_account from contract a "
                + " left join " + cmpDb + ".tenant t on a.tenant_id = t.id "
                + " where a.id = :id and a.is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Contract> list = this.list(Contract.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public Boolean remove(Long id) {
        String sql = "update contract set is_deleted = 1 where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }
}
