package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceCategory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * 服务类型数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class ServiceCategoryRepository extends JdbcGenericDao<ServiceCategory, Long> {

    public ServiceCategoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ServiceCategory> list(Pager pager) {
        String sql = "select a.* from service_category a where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return this.list(ServiceCategory.class, sql, paramMap);
    }

    public List<ServiceCategory> list(List<Param> params) {
        String sql = "select * from service_category a where a.is_deleted = 0  ";
        sql = getQueryBuilder().build(sql, params, null, "a");
        return this.list(ServiceCategory.class, sql, getQueryBuilder().getParam(params));
    }

    public List<ServiceCategory> listByService(Long serviceId) {
        String sql = "select a.* from service_category a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = new HashMap<>();
        if (null != serviceId && 0L != serviceId) {
            sql += " and a.service_id = :serviceId";
            paramMap.put("serviceId", serviceId);
        }
        return this.list(ServiceCategory.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from service_category a where a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        List<ServiceCategory> serviceCategories = this.list(ServiceCategory.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (ServiceCategory serviceCategory : serviceCategories) {
            SimpleBean bean = new SimpleBean(serviceCategory.getId(), serviceCategory.getName());
            beans.add(bean);
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from service_category a where a.is_deleted = 0  ";
        sql = getQueryBuilder().build(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id, Long userId) {
        String sql = "update service_category set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<ServiceCategory> listAll() {
        String sql = "select a.*,b.name service_name ,c.id catalog_id ,c.name catalog_name from service_category a " +
                " left join cloud_service b on a.service_id = b.id and b.is_deleted = 0 " +
                " left join service_catalog c on c.id = b.catalog_id and c.is_deleted = 0 where a.is_deleted = 0";
        return this.list(ServiceCategory.class, sql);
    }

    public ServiceCategory query(Long id) {
        String sql = "select * from service_category a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceCategory> list = this.list(ServiceCategory.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<ServiceCategory> listByServiceCode(String code) {
        String sql = "SELECT c.* FROM service_category c , cloud_service s "
                + " WHERE s.code = :code and s.id = c.service_id and c.is_deleted = 0 and s.is_deleted = 0 ";
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("code", code);
        return this.list(ServiceCategory.class, sql, paramMap);
    }

    public boolean existsName(String name, Long serviceId) {
        String sql = "select * from service_category a where a.is_deleted = 0 and a.name = :name and a.service_id = :serviceId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("serviceId", serviceId);
        List<ServiceCategory> list = this.list(ServiceCategory.class, sql, params);
        return !list.isEmpty();
    }

    public boolean existsSn(String sn, Long serviceId) {
        String sql = "select * from service_category a where a.is_deleted = 0 and a.sn = :sn and a.service_id = :serviceId";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        params.put("serviceId", serviceId);
        List<ServiceCategory> list = this.list(ServiceCategory.class, sql, params);
        return !list.isEmpty();
    }

    public Integer getMaxId() {
        String sql = "select max(a.id) from service_category a";
        Map<String, Object> paramMap = new HashMap<>();
        Long maxId = this.countQuery(sql, paramMap);
        return maxId == null ? 0 : maxId.intValue();
    }

}
