package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceOrder;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;

import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 订单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class ServiceOrderRepository extends BasicGenericDao<ServiceOrder, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public ServiceOrderRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ServiceOrder> list(Pager pager, Long userId, List<Long> projectIds, String uuid) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        String sql;
        if (StringUtils.isNotBlank(uuid)) {
            sql = "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id "
                    + " where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            sql = "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        if (null != userId && 0L != userId) {
            sql += " and ( a.user_id =:userId ";
            paramMap.put("userId", userId);
            if (null != projectIds && !projectIds.isEmpty()) {
                sql += " or a.project_id in (:projectIds)) ";
                paramMap.put("projectIds", projectIds);
            } else {
                sql += ")";
            }
        } else if (null != projectIds && !projectIds.isEmpty()) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public int count(List<Param> params, Long userId, List<Long> projectIds, String uuid) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        String sql;
        if (StringUtils.isNotBlank(uuid)) {
            sql = "select count(a.id)from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id "
                    + " where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            sql = "select count(a.id) from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        if (null != userId && 0L != userId) {
            sql += " and ( a.user_id =:userId ";
            paramMap.put("userId", userId);
            if (null != projectIds && !projectIds.isEmpty()) {
                sql += " or a.project_id in (:projectIds)) ";
                paramMap.put("projectIds", projectIds);
            } else {
                sql += ")";
            }
        } else if (null != projectIds && !projectIds.isEmpty()) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceOrder> list(Pager pager, String uuid) {
        String sql;
        if (StringUtils.isNotBlank(uuid)) {
            sql = "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id "
                    + " where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            sql = "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ServiceOrder.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        return this.list(new Pager(page, rows, params, sorter), null);
    }

    public int count(List<Param> params, String uuid) {
        String sql;
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        if (StringUtils.isNotBlank(uuid)) {
            sql = "select count(a.id) from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id "
                    + " where a.is_deleted = 0 and oi.sn like '%" + uuid + "%' ";
        } else {
            sql = "select count(a.id) from service_order a"
                    + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code and e.is_deleted = 0 "
                    + " left join " + cmpDb + ".project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "update service_order set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public Boolean batchRemove(List<Long> ids) {
        String sql = "update service_order set is_deleted = 1, gmt_modify = :gmtModify where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    /**
     * 查看租户下的服务订单(包括租户下用户的订单)
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public List<ServiceOrder> listByTenantId(Long tenantId) {
        String sql =
                "select a.*,b.name user_name,c.name tenant_name from service_order a left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id where a.is_deleted = 0  AND a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(ServiceOrder.class, sql, params);
    }

    public ServiceOrder query(Long id) {
        String sql =
                "select a.*,b.name user_name,c.name tenant_name,p.name project_name from service_order a left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id left join " + cmpDb + ".project p on p.id = a.project_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public ServiceOrder queryBySn(String sn) {
        String sql =
                "select a.*,b.name user_name,c.name tenant_name,s.name service_name from service_order a"
                        + " left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id"
                        + " left join cloud_service s on s.code = a.service_code where a.sn = :sn and s.is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ServiceOrder> orders(String chargeMode) {
        String sql = "select a.*,b.name user_name,c.name tenant_name from service_order a left join " + cmpDb + ".users b on b.id = a.user_id left join " + cmpDb + ".tenant c on c.id = a.tenant_id where a.charge_mode = :chargeMode";
        Map<String, Object> params = MapTools.simpleMap("chargeMode", chargeMode);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        if (!list.isEmpty()) {
            list = list.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(ServiceOrder::getId))), ArrayList::new));
        }
        return list;
    }

    public List<ServiceOrder> timeBetweenOrders(String start, String end, RequestContext context) {
        String sql = "select a.*,c.name service_name,u.name user_name , t.name tenant_name from service_order a " +
                " left join cloud_service c on a.service_code = c.code " +
                " left join " + cmpDb + ".users u on u.id = a.user_id " +
                " left join " + cmpDb + ".tenant t on t.id = a.tenant_id " +
                " where a.is_deleted = 0 and c.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        if (null != context && RequestContext.Catalog.Tenant == context.getCatalog()) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", context.getTenant());
        }
        if (null != context && RequestContext.Catalog.User == context.getCatalog()) {
            sql += " and a.user_id = :userId";
            params.put("userId", context.getTarget());
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<Map<String, Object>> top5(String start, String end, Boolean tenant, Boolean zone) {
        StringBuilder builder = new StringBuilder("select count(a.id) total,b.");
        if (null != tenant && tenant) {
            builder.append("name tenant_name");
        } else if (null != zone && zone) {
            builder.append("city");
        }
        builder.append(" from service_order a join tenant b on b.id = a.tenant_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != start) {
            builder.append(" and a.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (null != end) {
            builder.append(" and a.gmt_create <= :end");
            paramMap.put("end", end);
        }
        if (null != zone && zone) {
            builder.append(" and b.city is not null");
        }
        if (null != zone && zone) {
            builder.append(" group by b.city order by total desc limit 5");
        } else {
            builder.append(" group by a.tenant_id order by total desc limit 5");
        }
        return this.list(builder.toString(), paramMap);
    }

    public List<ServiceOrder> listByVendor(String start, String end, Long vendorId, RequestContext context) {
        String sql =
                "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join " + cmpDb + ".cloud_vendor cv on a.vendor_id = cv.id "
                        + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0 and cs.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (null != vendorId && 0 != vendorId) {
            sql += " and a.vendor_id = :vendorId";
            params.put("vendorId", vendorId);
        }
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        if (null != context) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", context.getTenant());
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByTenantId(String start, String end, Long tenantId) {
        String sql = "select a.*,cv.name vendor_name,cs.name service_name from service_order a " +
                " left join " + cmpDb + ".cloud_vendor cv on a.vendor_id = cv.id " +
                " left join cloud_service cs on cs.code=a.service_code left join " + cmpDb + ".tenant t on a.tenant_id = t.id " +
                " where a.is_deleted = 0 and cs.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByCode(String start, String end, String serviceCode) {
        String sql = "select a.*,cv.name vendor_name,cs.name service_name from service_order a " +
                " left join " + cmpDb + ".cloud_vendor cv on a.vendor_id = cv.id " +
                " left join cloud_service cs on cs.code=a.service_code " +
                " where a.is_deleted = 0  and cs.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotBlank(serviceCode)) {
            sql += " and a.service_code = :serviceCode";
            params.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByStatus(String start, String end, String status) {
        String sql = "select a.*,cv.name vendor_name,cs.name service_name from service_order a " +
                " left join " + cmpDb + ".cloud_vendor cv on a.vendor_id = cv.id " +
                " left join cloud_service cs on cs.code=a.service_code " +
                " where a.is_deleted = 0 and cs.is_deleted = 0 and cs.is_deleted = 0";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotBlank(status)) {
            sql += " and a.order_status = :status";
            params.put("status", status);
        }
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByType(String start, String end, String type) {
        String sql =
                "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join " + cmpDb + ".cloud_vendor cv on a.vendor_id = cv.id "
                        + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0 and cs.is_deleted = 0";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotBlank(type)) {
            sql += " and a.charge_mode = :type";
            params.put("type", type);
        }
        if (StringUtils.isNotBlank(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.isNotBlank(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public ServiceOrder queryOrderBySnAndType(String sn, String type) {
        String sql = "SELECT a.* FROM service_order a LEFT JOIN order_item b on a.id = b.order_id " +
                "WHERE b.sn = :sn AND a.order_type = :type";
        Map<String, Object> params = new HashMap<>();
        params.put("sn", sn);
        params.put("type", type);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return Optional.ofNullable(list).orElse(new ArrayList<>()).stream().findAny().orElse(null);
    }

    public List<Long> queryByChargeMode(String chargeMode) {
        String sql = "select * from service_order where charge_mode = '" + chargeMode + "' and is_deleted = 0";
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, null);
        if (list.isEmpty()) {
            return new ArrayList<>();
        }
        return list.stream().map(order -> order.getId()).collect(Collectors.toList());
    }
}
