package com.bocloud.cos.repository;

import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.megatron.database.core.intf.impl.BasicGenericDao;

import com.google.common.collect.Maps;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lym
 * @since 2022-06-16
 */
@Repository
public class VendorAccountRepository extends BasicGenericDao<CloudVendorAccount, Long> {

    public VendorAccountRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<CloudVendorAccount> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from cloud_vendor_account a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(CloudVendorAccount.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from cloud_vendor_account a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<CloudVendorAccount> listById(Long id) {
        StringBuilder sql = new StringBuilder();
        sql.append("select * from cloud_vendor_account where is_deleted = 0");
        if (null != id && id > 0L) {
            sql.append(" and id = ").append(id);
        }
        return this.list(CloudVendorAccount.class, sql.toString(), null);
    }

    public void remove(Long id) {
        String sql = "update cloud_vendor_account set is_deleted = 1 where id = " + id;
        this.execute(sql, null);
    }

    public List<CloudVendorAccount> listByTypes(List<String> types) {
        String sql = "select a.* from cloud_vendor_account a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = Maps.newHashMap();
        if (!ListTool.isEmpty(types)) {
            sql += " and a.type in (:types)";
            paramMap.put("types", types);
        }
        return this.list(CloudVendorAccount.class, sql, paramMap);
    }

    public List<CloudVendorAccount> queryVendorTypes() {
        String sql = "select vendor_type from cloud_vendor_account where is_deleted = 0 group by vendor_type";
        return this.list(CloudVendorAccount.class, sql, null);
    }
}
