package com.bocloud.cos.repository.assess;

import com.bocloud.cos.entity.assess.AssessConformityStandard;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 合格标准DAO
 *
 * @author lym
 */
@Repository
public class AssessConformityStandardRepository extends BasicGenericDao<AssessConformityStandard, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;
    public AssessConformityStandardRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<AssessConformityStandard> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.*,b.name classify_name,u.name creator from assess_conformity_standard a " +
                "left join assess_classify b on a.classify_id = b.id " +
                "LEFT JOIN " + cmpDb + ".users u ON a.creator_id = u.id " +
                "where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(AssessConformityStandard.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from assess_conformity_standard a where a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public int duplicateName(String name) {
        String sql = "select count(1) from assess_conformity_standard a where a.name = :name and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.NAME, name);
        return this.countQuery(sql, params).intValue();
    }

    public AssessConformityStandard getByTaskId(Long taskId) {
        String sql = "select * from assess_conformity_standard where id = (select conformity_standard_id from assess_task where id = " + taskId + ")";
        List<AssessConformityStandard> list = list(AssessConformityStandard.class, sql);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }
}
