package com.bocloud.cos.repository.assess;

import com.bocloud.cos.entity.assess.AssessMetric;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 考核指标DAO
 *
 * @author lym
 */
@Repository
public class AssessMetricRepository extends BasicGenericDao<AssessMetric, Long> {

    public AssessMetricRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<AssessMetric> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.*, b.name as category_name, (select count(1) from assess_rule where is_deleted = 0 and metric_id = a.id) as rule_count" +
                " from assess_metric a" +
                " left join assess_metric_classify b on a.metric_classify_id = b.id" +
                " where a.is_deleted = 0" +
                " and b.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(AssessMetric.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from assess_metric ai left join assess_metric_classify ac on ai.metric_classify_id = ac.id where ai.is_deleted = 0 and ac.is_deleted = 0";
        sql = getQueryBuilder().build(sql, params, null, "ai");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<AssessMetric> queryNotBindTaskByClassifyAndTaskId(Long classifyId, Long taskId) throws DataBaseException {
        String sql = "select * from assess_metric" +
                " where classify_id = " + classifyId +
                " and id in (select metric_id from assess_rule) " +
                " and id not in (select metric_id from assess_task_metric where task_id = " + taskId + ")";
        return this.list(AssessMetric.class, sql);
    }

    public List<AssessMetric> queryByTaskId(Long taskId) throws DataBaseException {
        String sql = "select * from assess_metric" +
                " where id in (select metric_id from assess_task_metric where task_id = " + taskId + ")";
        return this.list(AssessMetric.class, sql);
    }

}
