package com.bocloud.cos.operate;

import com.bocloud.cos.enums.ResourceCategory;
import com.google.common.collect.Lists;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 服务编码和资源类型的映射关系
 *
 * @author zhangyf
 * @since 2021-12-02
 */
public class ServiceResourceMapping {

    private static final Map<ResourceCategory, List<String>> MAPPING = new HashMap<>();

    public static Map<ResourceCategory, List<String>> getMapping() {
        return MAPPING;
    }

    static {
        MAPPING.put(ResourceCategory.MySQL, Lists.newArrayList("rds.mysql"));
        MAPPING.put(ResourceCategory.SqlServer, Lists.newArrayList("rds.sqlserver"));
        MAPPING.put(ResourceCategory.Redis, Lists.newArrayList("kvs.redis"));
        MAPPING.put(ResourceCategory.MongoDB, Lists.newArrayList("kvs.mongodb"));
        MAPPING.put(ResourceCategory.Kafka, Lists.newArrayList("cms.kafka"));
        MAPPING.put(ResourceCategory.RabbitMQ, Lists.newArrayList("cms.rabbitmq"));
        MAPPING.put(ResourceCategory.PostgreSQL, Lists.newArrayList("rds.postgresql"));
        MAPPING.put(ResourceCategory.Vpc, Lists.newArrayList(
                "vmware.nsx.vpc",
                "openstack.standard.vpc",
                "tce.standard.vpc",
                "manageone.standard.vpc",
                "huawei.standard.vpc",
                "tencent.standard.vpc",
                "aliyun.standard.vpc",
                "azure.standard.vpc",
                "aws.standard.vpc",
                "easystack.standard.vpc",
                "apsarastack.standard.vpc",
                "hcso.standard.vpc",
                "zstack.standard.vpc",
                "cloudos.standard.vpc",
                "kingcloud.standard.vpc",
                "qingcloud.standard.vpc"));
        MAPPING.put(ResourceCategory.LoadBalance, Lists.newArrayList(
                "network.aliyun.slb",
                "network.aws.slb",
                "hcso.standard.slb"));

        MAPPING.put(ResourceCategory.FloatingIp, Lists.newArrayList(
                "manageone.standard.eip",
                "openstack.standard.eip",
                "fusioncloud.standard.eip",
                "network.aliyun.eip",
                "network.huawei.eip",
                "network.tencent.eip",
                "network.tce.eip",
                "network.qcloud.eip",
                "network.aws.eip",
                "easystack.standard.eip",
                "hcso.standard.eip",
                "kingcloud.standard.eip"
        ));
        MAPPING.put(ResourceCategory.Computer, Lists.newArrayList("vmware.standard.server",
                "smartx.standard.server",
                "openstack.standard.server",
                "aliyun.standard.server",
                "huawei.standard.server",
                "fusioncloud.standard.server",
                "manageone.standard.server",
                "tce.standard.server",
                "tencent.standard.server",
                "azure.standard.server",
                "aws.standard.server",
                "fusionsphere.standard.server",
                "cas.standard.server",
                "easystack.standard.server",
                "apsarastack.standard.server",
                "hwaccess.standard.server",
                "hcso.standard.server",
                "zstack.standard.server",
                "cloudos.standard.server",
                "kingcloud.standard.server",
                "kvm.standard.server",
                "cnware.standard.server",
                "usphere.standard.server",
                "qingcloud.standard.server"));
        MAPPING.put(ResourceCategory.Storage, Lists.newArrayList("vmware.storage.disk",
                "openstack.standard.volume",
                "smartx.standard.volume",
                "manageone.standard.volume",
                "fusioncloud.standard.volume",
                "aliyun.standard.volume",
                "tce.standard.volume",
                "huawei.standard.volume",
                "tencent.standard.volume",
                "azure.standard.volume",
                "aws.standard.volume",
                "fusionsphere.standard.volume",
                "cas.standard.volume",
                "easystack.standard.volume",
                "apsarastack.standard.volume",
                "hcso.standard.volume",
                "hwaccess.standard.volume",
                "zstack.standard.volume",
                "cloudos.standard.volume",
                "kingcloud.standard.volume",
                "kvm.standard.volume",
                "usphere.standard.volume",
                "cnware.standard.volume",
                "qingcloud.standard.volume"
        ));
    }

    public static ResourceCategory getResourceCategory(String serviceCode) {
        for (Map.Entry<ResourceCategory, List<String>> m : MAPPING.entrySet()) {
            if (m.getValue().contains(serviceCode)) {
                return m.getKey();
            }
        }
        throw new IllegalArgumentException("unknown service code: " + serviceCode);
    }

}
