package com.bocloud.cos.provider.aws;

import com.bocloud.cos.driver.SuperProvider;
import com.bocloud.cos.enums.AwsServiceEnum;
import com.bocloud.cos.model.Butler;
import fr.opensagres.xdocreport.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.AwsCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.SdkClient;
import software.amazon.awssdk.http.SdkHttpClient;
import software.amazon.awssdk.http.apache.ApacheHttpClient;
import software.amazon.awssdk.http.apache.ProxyConfiguration;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.S3Configuration;

import java.net.URI;

public class AwsProvider implements SuperProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsProvider.class);

    private final Butler butler;
    private String regionId = "cn-north-1";// 可用域
    private final String accessKey;// access key
    private final String secretKey;// secret access key
    private final AwsCredentials credentials;// 访问凭证

    public AwsProvider(Butler butler) {
        this.regionId = butler.getRegionId();
        this.accessKey = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.butler = butler;
        this.credentials = AwsBasicCredentials.create(this.accessKey, this.secretKey);
    }

    public SdkClient getClient(AwsServiceEnum awsService) {
        SdkHttpClient httpClient = null;
        if (StringUtils.isNotEmpty(butler.getProxyUri())) {
            httpClient = ApacheHttpClient.builder()
                    .proxyConfiguration(ProxyConfiguration.builder().endpoint(URI.create(butler.getProxyUri())).build())
                    .build();
        }
        SdkClient sdkClient = null;
        switch (awsService) {
            case EC2:
                sdkClient = Ec2Client.builder().region(Region.of(regionId))
                        .credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .build();
                break;
            case S3:
                sdkClient = S3Client.builder().region(Region.of(regionId)).
                        credentialsProvider(StaticCredentialsProvider.create(this.credentials)).httpClient(httpClient)
                        .serviceConfiguration(S3Configuration.builder().useArnRegionEnabled(true).build())
                        .build();
                break;
            default:
                break;
        }
        return sdkClient;
    }

    @Override
    public boolean accessable() {
        boolean flag;
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsServiceEnum.EC2);
            client.describeAvailabilityZones();
            flag = true;
        } catch (Exception e) {
            logger.error("aws accessable error :", e);
            flag = false;
        }
        return flag;
    }

    public Butler getButler() {
        return butler;
    }
}