package com.bocloud.cos.provider.azure;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.model.AzureCloudBillModel;
import com.bocloud.cos.model.BillPeriodModel;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.BillingProvider;
import com.bocloud.cos.provider.convertor.BeanJsonConvertor;
import com.megatron.common.model.GeneralResult;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
@Provider("bocloud.cmp.azure")
public class AzureBillProvider extends AzureRestApiProvider implements BillingProvider {
    private BeanJsonConvertor<PublicCloudBillModel> billConsumptionConvertor = new BillConsumptionConvertor();
    private BeanJsonConvertor<BillPeriodModel> billPeriodConvertor = new BillPeriodConvertor();

    public AzureBillProvider(Butler butler) {
        super(butler);
    }

    @Override
    public GeneralResult billPeriods() {
        try {
            String api = "/subscriptions/%s/providers/Microsoft.Billing/billingPeriods?api-version=2017-04-24-preview";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<BillPeriodModel> billPeriodModels = this.billPeriodConvertor.convertList(jsonObject.getJSONArray("value"));
            log.info("Azure 底层获取计费周期API成功！");
            return new GeneralResult(true, billPeriodModels, "成功！");
        } catch (Exception var6) {
            log.error("billPeriods error :", var6);
            return new GeneralResult(false, var6.getMessage());
        }
    }

    @Override
    public GeneralResult periodPrices(String periodUuid) {
        try {
            AzureCloudBillModel model = new AzureCloudBillModel();
            String api = "%s/providers/Microsoft.Consumption/usageDetails?api-version=2019-10-01";
            String url = String.format(api, periodUuid);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PublicCloudBillModel> billModels = this.billConsumptionConvertor.convertList(jsonObject.getJSONArray("value"));
            String nextLink = jsonObject.getString("nextLink");
            model.setModels(billModels);
            model.setNextLink(nextLink);
            log.info("Azure 底层获取账单API成功！");
            return new GeneralResult(true, model, "成功!");
        } catch (Exception var9) {
            log.error("periodPrices error :", var9);
            return new GeneralResult(false, var9.getMessage());
        }
    }

    @Override
    public GeneralResult nextLinkPrices(String nextLink) {
        try {
            AzureCloudBillModel model = new AzureCloudBillModel();
            String response = this.requestNoEndpoint(RequestStatus.GET, nextLink, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PublicCloudBillModel> billModels = this.billConsumptionConvertor.convertList(jsonObject.getJSONArray("value"));
            String newNextLink = jsonObject.getString("nextLink");
            model.setModels(billModels);
            model.setNextLink(newNextLink);
            log.info("Azure 底层分页账单API成功！");
            return new GeneralResult(true, model, "成功!");
        } catch (Exception var7) {
            log.error("nextLinkPrices error :", var7);
            return new GeneralResult(false, var7.getMessage());
        }
    }

}
