package com.bocloud.cos.provider.azure;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.driver.SuperProvider;
import com.bocloud.cos.model.Butler;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import com.megatron.common.utils.MapTools;
import com.microsoft.azure.AzureEnvironment;
import lombok.Data;
import org.apache.commons.io.IOUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Provider("bocloud.cmp.azure")
public class AzureRestApiProvider implements SuperProvider {

    private static final Logger LOGGER = LoggerFactory.getLogger(AzureRestApiProvider.class);
    private static final Integer TIMEOUT = 1200000;
    private String subscriptionId;
    private String clientId;
    private String clientSecret;
    private String tenantId;
    private String version;
    private Butler butler;
    private AzureRestApiProvider.AzureToken token;
    private HttpClient client;
    private AzureEnvironment azureEnvironment;

    public AzureRestApiProvider(Butler butler) {
        this.tenantId = butler.getTenant();
        this.subscriptionId = butler.getSubscriptionId();
        this.clientId = butler.getClientId();
        this.clientSecret = butler.getSecretKey();
        this.butler = butler;
        this.version = butler.getVersion();
        String var2 = this.version;
        byte var3 = -1;
        switch (var2.hashCode()) {
            case -1868435358:
                if (var2.equals("AZURE_US_GOVERNMENT")) {
                    var3 = 2;
                }
                break;
            case -1670362685:
                if (var2.equals("AZURE_GERMANY")) {
                    var3 = 3;
                }
                break;
            case 62794351:
                if (var2.equals("AZURE")) {
                    var3 = 0;
                }
                break;
            case 414759783:
                if (var2.equals("AZURE_CHINA")) {
                    var3 = 1;
                }
        }

        switch (var3) {
            case 0:
                this.azureEnvironment = AzureEnvironment.AZURE;
                break;
            case 1:
                this.azureEnvironment = AzureEnvironment.AZURE_CHINA;
                break;
            case 2:
                this.azureEnvironment = AzureEnvironment.AZURE_US_GOVERNMENT;
                break;
            case 3:
                this.azureEnvironment = AzureEnvironment.AZURE_GERMANY;
                break;
            default:
                this.azureEnvironment = AzureEnvironment.AZURE_CHINA;
        }

        this.token = this.getToken(this.tenantId, this.clientId, this.clientSecret);
    }

    @Override
    public boolean accessable() {
        boolean flag = this.getToken() != null;
        return flag;
    }

    public String request(AzureRestApiProvider.RequestStatus method, String url, String params) {
        Result result = this.connection(method, url, null, JSON.parseObject(params, Map.class), true);
        if (result.isFailed()) {
            throw new IllegalStateException("[" + this.unicodeToString(result.getErrorMsg()) + "]" + this.unicodeToString(result.getMessage()));
        } else {
            return this.unicodeToString(result.getMessage());
        }
    }

    public String requestNoEndpoint(AzureRestApiProvider.RequestStatus method, String url, String params) {
        Result result = this.connection(method, url, null, JSON.parseObject(params, Map.class), false);
        if (result.isFailed()) {
            throw new IllegalStateException("[" + this.unicodeToString(result.getErrorMsg()) + "]" + this.unicodeToString(result.getMessage()));
        } else {
            return this.unicodeToString(result.getMessage());
        }
    }

    public Result connection(AzureRestApiProvider.RequestStatus method, String url, Map<String, Object> header, Map<String, Object> params, Boolean isEndpoint) {
        if (MapTools.isEmpty(header)) {
            header = this.buildHeader();
        }

        if (isEndpoint) {
            url = this.azureEnvironment.endpoints().get("resourceManagerEndpointUrl") + url;
        }

        LOGGER.debug(url);
        Result result;
        switch (method) {
            case GET:
                result = this.get(url, header, params);
                break;
            case PUT:
                result = this.put(url, header, params);
                break;
            case POST:
                result = this.post(url, header, params);
                break;
            case DELETE:
                result = this.delete(url, header, params);
                break;
            case PATCH:
                result = this.patch(url, header, params);
                break;
            default:
                result = new Result(false, "请求方式不存在！", "请求方式不存在！");
        }

        return result;
    }

    public Result get(String url, Map<String, Object> header, Map<String, Object> params) {
        this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);

        Result var6;
        try {
            Map<String, Object> auth = new HashMap();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }

            this.client.setDataFormat(null);
            Result result = this.client.get(auth, params, url);
            this.client.close();
            var6 = result;
        } finally {
            this.client.close();
        }

        return var6;
    }

    public Result put(String url, Map<String, Object> header, Map<String, Object> params) {
        this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);

        Result var6;
        try {
            Map<String, Object> auth = new HashMap();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }

            this.client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.put(auth, params, url);
            this.client.close();
            var6 = result;
        } finally {
            this.client.close();
        }

        return var6;
    }

    public Result post(String url, Map<String, Object> header, Map<String, Object> params) {
        this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);

        Result var6;
        try {
            Map<String, Object> auth = new HashMap();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }

            this.client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.post(auth, params, url);
            this.client.close();
            var6 = result;
        } finally {
            this.client.close();
        }

        return var6;
    }

    public Result patch(String url, Map<String, Object> header, Map<String, Object> params) {
        this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);

        Result var6;
        try {
            Map<String, Object> auth = new HashMap();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }

            this.client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.patch(auth, params, url);
            this.client.close();
            var6 = result;
        } finally {
            this.client.close();
        }

        return var6;
    }

    public Result delete(String url, Map<String, Object> header, Map<String, Object> params) {
        this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);

        Result var6;
        try {
            Map<String, Object> auth = new HashMap();
            auth.put("Authorization", this.getToken().getToken_type() + " " + this.getToken().getAccess_token());
            if (null != header) {
                auth.putAll(header);
            }

            this.client.setDataFormat(PostDataFormat.RAW);
            Result result = this.client.delete(auth, params, url);
            this.client.close();
            var6 = result;
        } finally {
            this.client.close();
        }

        return var6;
    }

    private String unicodeToString(String string) {
        if (StringUtils.isEmpty(string)) {
            return string;
        } else {
            StringBuffer stringBuffer = new StringBuffer();
            String pattern = "\\\\u[0-9,a-f,A-F]{4}";
            Matcher matcher = Pattern.compile(pattern).matcher(string);

            while (matcher.find()) {
                char c = (char) Integer.parseInt(matcher.group().replaceAll("\\\\u", ""), 16);
                matcher.appendReplacement(stringBuffer, new String(new char[]{c}));
            }

            matcher.appendTail(stringBuffer);
            return stringBuffer.toString();
        }
    }

    public AzureRestApiProvider.AzureToken getToken(String tenantId, String clientId, String clientSecret) {
        AzureRestApiProvider.AzureToken token = null;

        try {
            String url = this.azureEnvironment.endpoints().get("activeDirectoryEndpointUrl") + tenantId + "/oauth2/v2.0/token";
            this.client = new HttpClient(TIMEOUT, PostDataFormat.RAW);
            HttpPost httpPost = new HttpPost(url);
            Map<String, Object> header = new HashMap();
            header.put("content-type", "application/x-www-form-urlencoded");
            this.client.buildHeader(httpPost, header);
            Map<String, Object> queryString = new HashMap();
            queryString.put("grant_type", "client_credentials");
            queryString.put("scope", this.azureEnvironment.endpoints().get("resourceManagerEndpointUrl") + ".default");
            queryString.put("client_id", clientId);
            queryString.put("client_secret", clientSecret);
            queryString.put("tenant", tenantId);
            httpPost.setEntity(this.getEntityConvertor(queryString));
            CloseableHttpResponse response = this.client.getHttpClient().execute(httpPost);
            token = JSONObject.parseObject(AzureRestApiProvider.Respone2Content.getBody(response).toJSONString(), AzureRestApiProvider.AzureToken.class);
        } catch (Exception var13) {
            LOGGER.error("AZURE令牌请求失败:" + var13.getMessage());
        } finally {
            this.client.close();
        }

        return token;
    }

    private UrlEncodedFormEntity getEntityConvertor(Map<String, Object> queryString) throws Exception {
        List<NameValuePair> params = new ArrayList();
        Iterator var3 = queryString.entrySet().iterator();

        while (var3.hasNext()) {
            Map.Entry<String, Object> entry = (Map.Entry) var3.next();
            NameValuePair nameValuePair = new BasicNameValuePair(entry.getKey(), String.valueOf(entry.getValue()));
            params.add(nameValuePair);
        }

        UrlEncodedFormEntity urlEncodedFormEntity = new UrlEncodedFormEntity(params, "UTF-8");
        return urlEncodedFormEntity;
    }

    private Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap();
        header.put("Content-Type", "application/json");
        return header;
    }

    public String getVersion() {
        return this.version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getSubscriptionId() {
        return this.subscriptionId;
    }

    public Butler getButler() {
        return this.butler;
    }

    public void setButler(Butler butler) {
        this.butler = butler;
    }

    public AzureRestApiProvider.AzureToken getToken() {
        return this.token;
    }

    public enum RequestStatus {
        GET,
        POST,
        DELETE,
        PUT,
        PATCH;

        RequestStatus() {
        }
    }

    public static class Respone2Content {
        public Respone2Content() {
        }

        public static JSONObject getBody(CloseableHttpResponse httpResponse) throws Exception {
            JSONObject content = JSONObject.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "UTF-8"));
            return content;
        }
    }

    @Data
    public static class AzureToken {
        private String token_type;
        private String expires_in;
        private String ext_expires_in;
        private String expires_on;
        private String not_before;
        private String resource;
        private String access_token;

        public AzureToken() {
        }
    }
}
