package com.bocloud.cos.provider.azure;

import com.alibaba.fastjson.JSONObject;
import com.megatron.common.utils.DateTools;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.convertor.BeanJsonConvertor;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Optional;

public class BillConsumptionConvertor implements BeanJsonConvertor<PublicCloudBillModel> {

    public BillConsumptionConvertor() {
    }

    @Override
    public PublicCloudBillModel convertModel(JSONObject object) {
        return this.consume(object);
    }

    public PublicCloudBillModel consume(JSONObject object) {
        PublicCloudBillModel model = new PublicCloudBillModel();
        model.setOrderSn(object.getString("id").toLowerCase());
        Optional.ofNullable(object.getJSONObject("properties")).ifPresent((properties) -> {
            Optional.ofNullable(properties.getString("subscriptionId")).ifPresent(model::setSubscriptionId);
            Optional.ofNullable(properties.getString("subscriptionName")).ifPresent(model::setSubscriptionName);
            Optional.ofNullable(properties.getString("resourceGroup")).ifPresent(model::setResourceGroupName);
            Optional.ofNullable(properties.getString("product")).ifPresent(model::setProductName);
            Optional.ofNullable(properties.getString("resourceId")).ifPresent(model::setResourceId);
            Optional.ofNullable(properties.getString("resourceName")).ifPresent(model::setResourceName);
            Optional.ofNullable(properties.getString("resourceId")).ifPresent((resourceId) -> {
                String[] split = resourceId.split("/");
                String productLine;
                if (split.length > 7) {
                    productLine = split[6] + "/" + split[7];
                } else {
                    productLine = resourceId;
                }

                model.setProductLine(productLine);
            });
            Optional.ofNullable(properties.getString("resourceLocation")).ifPresent(model::setRegionId);
            Optional.ofNullable(properties.getString("cost")).map((cost) -> {
                return new BigDecimal(cost);
            }).ifPresent(model::setMoney);
            Optional.ofNullable(properties.getString("unitPrice")).map((unitPrice) -> {
                return new BigDecimal(unitPrice);
            }).ifPresent(model::setUnitPrice);
            Optional.ofNullable(properties.getString("quantity")).map((quantity) -> {
                return new BigDecimal(quantity);
            }).ifPresent(model::setQuantity);
            Optional.ofNullable(properties.getString("billingCurrency")).ifPresent(model::setBillingCurrency);
            Optional.ofNullable(properties.getString("billingPeriodStartDate")).map((billingPeriodStartDate) -> {
                return DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodStartDate, "Z");
            }).ifPresent(model::setFeeBeginTime);
            Optional.ofNullable(properties.getString("billingPeriodEndDate")).map((billingPeriodEndDate) -> {
                return DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodEndDate, "Z");
            }).ifPresent(model::setFeeEndTime);
            Optional.ofNullable(properties.getString("date")).map((date) -> {
                return DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", date, "Z");
            }).ifPresent(model::setUseResourceData);
            Optional.ofNullable(properties.getString("consumedService")).ifPresent(model::setConsumedService);
            Optional.ofNullable(properties.getString("billingPeriodStartDate")).map((billingPeriodStartDate) -> {
                return DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodStartDate, "Z");
            }).map(this::getTime).ifPresent(model::setMonth);
            model.setMeterId(properties.getString("meterId"));
            Optional.ofNullable(properties.getString("cost")).map((cost) -> {
                return new BigDecimal(cost);
            }).ifPresent(model::setPretaxAmount);
            model.setPretaxGrossAmount(model.getQuantity().multiply(model.getUnitPrice()));
            Optional.ofNullable(properties.getJSONObject("meterDetails")).ifPresent((meterDetails) -> {
                model.getClass();
                PublicCloudBillModel.MeterDetailsModel detailsModel = new PublicCloudBillModel.MeterDetailsModel();
                detailsModel.setMeterName(meterDetails.getString("meterName"));
                detailsModel.setMeterCategory(meterDetails.getString("meterCategory"));
                detailsModel.setMeterSubCategory(meterDetails.getString("meterSubCategory"));
                detailsModel.setUnitOfMeasure(meterDetails.getString("unitOfMeasure"));
                model.setMeterDetailsModel(detailsModel);
            });
        });
        return model;
    }

    private String getTime(Date date) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");
        String time = dateFormat.format(date);
        return time;
    }
}
