package com.bocloud.cos.provider.huawei;

import com.megatron.common.utils.DateTools;
import com.megatron.common.utils.ListTool;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.bss.v2.model.MonthlyBillRecord;
import com.huaweicloud.sdk.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;

@Slf4j
@Component
public class BillingConvertor implements BeanConvertor<PublicCloudBillModel, MonthlyBillRecord> {
    public BillingConvertor() {
    }

    @Override
    public List<PublicCloudBillModel> convertList(List<? extends MonthlyBillRecord> models) {
        List<PublicCloudBillModel> list = new ArrayList();
        if (ListTool.isEmpty(models)) {
            return list;
        } else {
            Iterator var3 = models.iterator();

            while (var3.hasNext()) {
                MonthlyBillRecord model = (MonthlyBillRecord) var3.next();
                list.add(this.convertModel(model));
            }

            return list;
        }
    }

    @Override
    public PublicCloudBillModel convertModel(MonthlyBillRecord model) {
        Map<Integer, String> chargeTypeMap = new HashMap(8);
        chargeTypeMap.put(1, "包年/包月");
        chargeTypeMap.put(3, "按需");
        chargeTypeMap.put(10, "预留实例");
        Map<Integer, String> billTypeMap = new HashMap(8);
        billTypeMap.put(1, "消费-新购");
        billTypeMap.put(2, "消费-续订");
        billTypeMap.put(3, "消费-变更");
        billTypeMap.put(4, "退款-退订");
        billTypeMap.put(5, "消费-使用");
        billTypeMap.put(8, "消费-自动续订");
        billTypeMap.put(9, "调账-补偿");
        billTypeMap.put(14, "消费-服务支持计划月末扣费");
        billTypeMap.put(16, "调账-扣费");
        Map<Integer, String> statusMap = new HashMap(8);
        statusMap.put(1, "已支付");
        statusMap.put(2, "未清算");
        statusMap.put(3, "未结算");
        PublicCloudBillModel publicCloudBillModel = new PublicCloudBillModel();
        if (model != null) {
            publicCloudBillModel.setMonth(model.getBillCycle());
            publicCloudBillModel.setProductCode(model.getServiceTypeCode());
            publicCloudBillModel.setResourceId(model.getResourceTypeCode());
            publicCloudBillModel.setRegionId(model.getRegionCode());
            if (null != model.getChargingMode()) {
                publicCloudBillModel.setSubscriptionId(String.valueOf(model.getChargingMode()));
            }

            if (null != model.getChargingMode() && chargeTypeMap.containsKey(model.getChargingMode())) {
                publicCloudBillModel.setSubscriptionType(chargeTypeMap.get(model.getChargingMode()));
            }

            String consumeTime = model.getConsumeTime();
            if (!StringUtils.isEmpty(consumeTime)) {
                if (consumeTime.contains("Z-")) {
                    String[] split = consumeTime.split("Z-");
                    if (null != split && split.length > 0 && !StringUtils.isEmpty(split[0])) {
                        publicCloudBillModel.setFeeBeginTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", split[0] + "Z", "Z"));
                    }

                    if (null != split && split.length > 1 && !StringUtils.isEmpty(split[1])) {
                        publicCloudBillModel.setFeeEndTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", split[1], "Z"));
                    }
                } else {
                    publicCloudBillModel.setFeeBeginTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", consumeTime + "Z", "Z"));
                    publicCloudBillModel.setFeeEndTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", consumeTime, "Z"));
                }
            }

            if (!StringUtils.isEmpty(model.getTradeTime())) {
                Date transfer = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", model.getTradeTime(), "Z");
                if (null != transfer) {
                    publicCloudBillModel.setPayTime(String.valueOf(transfer));
                }
            }

            publicCloudBillModel.setOrderSn(model.getTradeId());
            if (null != model.getBillType() && billTypeMap.containsKey(model.getBillType())) {
                publicCloudBillModel.setReduceType(billTypeMap.get(model.getBillType()));
            }

            if (null != model.getStatus() && statusMap.containsKey(model.getStatus())) {
                publicCloudBillModel.setStatus(statusMap.get(model.getStatus()));
            }

            if (null != model.getOfficialAmount()) {
                publicCloudBillModel.setPretaxGrossAmount(BigDecimal.valueOf(model.getOfficialAmount()));
            }

            if (null != model.getEraseAmount()) {
                publicCloudBillModel.setRoundDownDiscount(BigDecimal.valueOf(model.getEraseAmount()));
            }

            if (null != model.getConsumeAmount()) {
                publicCloudBillModel.setMoney(BigDecimal.valueOf(model.getConsumeAmount()));
            }

            if (null != model.getCouponAmount()) {
                publicCloudBillModel.setDeductedByCashCoupons(BigDecimal.valueOf(model.getCouponAmount()));
            }

            if (null != model.getFlexipurchaseCouponAmount()) {
                publicCloudBillModel.setDeductedByCoupons(BigDecimal.valueOf(model.getFlexipurchaseCouponAmount()));
            }

            if (null != model.getStoredValueCardAmount()) {
                publicCloudBillModel.setDeductedByPrepaidCard(BigDecimal.valueOf(model.getStoredValueCardAmount()));
            }

            if (null != model.getDebtAmount()) {
                publicCloudBillModel.setOutstandingAmount(BigDecimal.valueOf(model.getDebtAmount()));
            }

            publicCloudBillModel.setRegionName(model.getRegionName());
            publicCloudBillModel.setBillingCurrency("RMB");
        }

        return publicCloudBillModel;
    }
}