package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.enums.CloudProvider;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.PmsInternalService;
import com.bocloud.cos.intf.ResourceAnalysisService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.ListUtils;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.utils.PredictUtils;
import com.bocloud.cos.entity.internel.AllCloudCount;
import com.bocloud.cos.entity.internel.CloudResourceCapacity;
import com.bocloud.cos.entity.internel.CloudServer;
import com.bocloud.cos.entity.internel.CloudVendor;
import com.bocloud.cos.entity.internel.DataStore;
import com.bocloud.cos.entity.internel.FloatingIp;
import com.bocloud.cos.entity.internel.Ip;
import com.bocloud.cos.entity.internel.IpPool;
import com.bocloud.cos.entity.internel.NameAndValue;
import com.bocloud.cos.entity.internel.Server;
import com.bocloud.cos.entity.internel.ServerConfig;
import com.bocloud.cos.entity.internel.Volume;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringEscapeUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.net.URLDecoder;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;

/**
 * @author tyl
 * @time 2020/6/29 周一 17:04
 */
@Service
@Slf4j
public class ResourceAnalysisServiceImpl implements ResourceAnalysisService {

    private final ImsInternalService imsInternalService;
    private final PmsInternalService pmsInternalService;

    public ResourceAnalysisServiceImpl(ImsInternalService imsInternalService, PmsInternalService pmsInternalService) {
        this.imsInternalService = imsInternalService;
        this.pmsInternalService = pmsInternalService;
    }

    @Override
    public GeneralResult<JSONObject> resourceStatistics(Long vendorId, Long vdcId, Long clusterId, String condition) {
        if ("vendorCount".equals(condition)) {
            return getVendorCount(vendorId, vdcId);
        }
        List<CloudVendor> vendorsResult = imsInternalService.getVendorList();
        //获取平台数
        List<CloudVendor> cloudVendors = vendorsResult.stream().filter(cloudVendor -> {
            boolean flag = true;
            if (vendorId != null && vendorId != 0L) {
                flag = vendorId.equals(cloudVendor.getId());
            }
            if (vdcId != null && vdcId != 0L) {
                flag = flag && vdcId.equals(cloudVendor.getDcId());
            }
            return flag;
        }).collect(Collectors.toList());
        if ("hostCount".equals(condition) || "vmsCount".equals(condition)) {
            return getHostOrVmsCount(condition, cloudVendors);
        }
        Map<Long, String> vendorCache =
                cloudVendors.stream().collect(Collectors.toMap(CloudVendor::getId, CloudVendor::getName));
        if ("ipPoolCount".equals(condition)) {
            return getIpPoolCount(vendorId, vendorCache);
        }
        if ("dataStoreCount".equals(condition)) {
            return getDataStoreCount(vendorId, vdcId, vendorCache);
        }
        if ("volumeCount".equals(condition)) {
            return getVolumeCount(vendorId, vendorCache);
        }
        return GeneralResult.FAILED("不支持的查询类型");
    }

    public GeneralResult<JSONObject> getHostOrVmsCount(String condition, List<CloudVendor> cloudVendors) {
        JSONObject result = new JSONObject();
        JSONArray data = new JSONArray();
        for (CloudVendor cloudVendor : cloudVendors) {
            JSONObject jsonObject = new JSONObject();
            result.put("title", "hostCount".equals(condition) ? "宿主机数量分布" : "云主机数量分布");
            jsonObject.put("name", cloudVendor.getName());
            jsonObject.put("value",
                    "hostCount".equals(condition) ? Optional.ofNullable(cloudVendor.getHostCount()).orElse(0L) :
                            Optional.ofNullable(cloudVendor.getInstance()).orElse(0L));
            jsonObject.put("type", cloudVendor.getType());
            data.add(jsonObject);
        }
        result.put("data", data);
        return new GeneralResult<>(true, result, "查询成功");
    }

    public GeneralResult<JSONObject> getClusterCount(List<CloudVendor> cloudVendors) {
        JSONObject result = new JSONObject();
        JSONArray data = new JSONArray();
        for (CloudVendor cloudVendor : cloudVendors) {
            JSONObject jsonObject = new JSONObject();
            result.put("title", "集群数量分布");
            jsonObject.put("name", cloudVendor.getName());
            jsonObject.put("value", null == cloudVendor.getClusterCount() ? 0 : cloudVendor.getClusterCount());
            jsonObject.put("type", cloudVendor.getType());
            data.add(jsonObject);
        }
        result.put("data", data);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Override
    public GeneralResult<List<AllCloudCount>> getAllCloudCount(Long vendorId, Long vdcId, Long clusterId) {
        List<CloudVendor> vendorsResult = imsInternalService.getVendorList();
        //获取平台数
        List<CloudVendor> cloudVendors = vendorsResult.stream().filter(cloudVendor -> {
            boolean flag = true;
            if (vendorId != null && vendorId != 0L) {
                flag = vendorId.equals(cloudVendor.getId());
            }
            if (vdcId != null && vdcId != 0L) {
                flag = flag && vdcId.equals(cloudVendor.getDcId());
            }
            return flag;
        }).collect(Collectors.toList());
        Map<Long, String> vendorCache =
                cloudVendors.stream().collect(Collectors.toMap(CloudVendor::getId, CloudVendor::getName));
        return getAllCloudCount(vendorId, vdcId, vendorCache, cloudVendors);
    }

    public GeneralResult<List<AllCloudCount>> getAllCloudCount(Long vendorId, Long vdcId, Map<Long, String> vendorCache,
                                                               List<CloudVendor> cloudVendors) {
        List<AllCloudCount> allCloudCounts = new ArrayList<>();
        try {
            GeneralResult<JSONObject> vendorCount = getVendorCount(vendorId, vdcId);
            GeneralResult<JSONObject> hostCount = getHostOrVmsCount("hostCount", cloudVendors);
            GeneralResult<JSONObject> vmsCount = getHostOrVmsCount("vmsCount", cloudVendors);
            GeneralResult<JSONObject> dataStoreCount = getDataStoreCount(vendorId, vdcId, vendorCache);
            GeneralResult<JSONObject> volumeCount = getVolumeCount(vendorId, vendorCache);
            GeneralResult<JSONObject> clusterCount = getClusterCount(cloudVendors);
            if (vendorCount.isSuccess() && null != vendorCount.getData()) {
                JSONArray data = vendorCount.getData().getJSONArray("data");
                List<NameAndValue> nameAndValues =
                        JSON.parseArray(URLDecoder.decode(String.valueOf(data), "UTF-8"), NameAndValue.class);
                nameAndValues.stream().forEach(nameAndValue -> {
                    AllCloudCount allCloudCount = new AllCloudCount();
                    allCloudCount.setVendorName(nameAndValue.getName());
                    allCloudCounts.add(allCloudCount);
                });

                if (hostCount.isSuccess() && null != hostCount.getData()) {
                    JSONArray hostArray = hostCount.getData().getJSONArray("data");
                    List<NameAndValue> nameAndValues1 =
                            JSON.parseArray(URLDecoder.decode(String.valueOf(hostArray), "UTF-8"), NameAndValue.class);
                    nameAndValues1.stream().forEach(nameAndValue -> {
                        allCloudCounts.stream().forEach(allCloudCount -> {
                            if (allCloudCount.getVendorName().equals(nameAndValue.getType())) {
                                allCloudCount.setHostCount(allCloudCount.getHostCount() + nameAndValue.getValue());
                            }
                        });
                    });
                }
                if (vmsCount.isSuccess() && null != vmsCount.getData()) {
                    JSONArray vmsArray = vmsCount.getData().getJSONArray("data");
                    List<NameAndValue> nameAndValues1 =
                            JSON.parseArray(URLDecoder.decode(String.valueOf(vmsArray), "UTF-8"), NameAndValue.class);
                    nameAndValues1.stream().forEach(nameAndValue -> {
                        allCloudCounts.stream().forEach(allCloudCount -> {
                            if (allCloudCount.getVendorName().equals(nameAndValue.getType())) {
                                allCloudCount.setVmsCount(allCloudCount.getVmsCount() + nameAndValue.getValue());
                            }
                        });
                    });
                }
                if (dataStoreCount.isSuccess() && null != dataStoreCount.getData()) {
                    JSONArray dataStoreArray = dataStoreCount.getData().getJSONArray("data");
                    List<NameAndValue> nameAndValues1 =
                            JSON.parseArray(URLDecoder.decode(String.valueOf(dataStoreArray), "UTF-8"), NameAndValue.class);
                    nameAndValues1.stream().forEach(nameAndValue -> {
                        allCloudCounts.stream().forEach(allCloudCount -> {
                            cloudVendors.stream().forEach(cloudVendor -> {
                                if (cloudVendor.getName().equals(nameAndValue.getName()) && allCloudCount
                                        .getVendorName().equals(cloudVendor.getType())) {
                                    allCloudCount
                                            .setDataStoreCount(allCloudCount.getDataStoreCount() + nameAndValue.getValue());
                                }
                            });
                        });
                    });
                }
                if (volumeCount.isSuccess() && null != volumeCount.getData()) {
                    JSONArray volumeArray = volumeCount.getData().getJSONArray("data");
                    List<NameAndValue> nameAndValues1 =
                            JSON.parseArray(URLDecoder.decode(String.valueOf(volumeArray), "UTF-8"), NameAndValue.class);
                    nameAndValues1.stream().forEach(nameAndValue -> {
                        allCloudCounts.stream().forEach(allCloudCount -> {
                            cloudVendors.stream().forEach(cloudVendor -> {
                                if (cloudVendor.getName().equals(nameAndValue.getName()) && allCloudCount
                                        .getVendorName().equals(cloudVendor.getType())) {
                                    allCloudCount
                                            .setVolumeCount(allCloudCount.getVolumeCount() + nameAndValue.getValue());
                                }
                            });
                        });
                    });
                }
                if (clusterCount.isSuccess() && null != clusterCount.getData()) {
                    JSONArray clusterArray = clusterCount.getData().getJSONArray("data");
                    List<NameAndValue> nameAndValues1 =
                            JSON.parseArray(URLDecoder.decode(String.valueOf(clusterArray), "UTF-8"), NameAndValue.class);
                    nameAndValues1.stream().forEach(nameAndValue -> {
                        allCloudCounts.stream().forEach(allCloudCount -> {
                            if (allCloudCount.getVendorName().equals(nameAndValue.getType())) {
                                allCloudCount
                                        .setClusterCount(allCloudCount.getClusterCount() + nameAndValue.getValue());
                            }
                        });
                    });
                }
                return new GeneralResult<>(true, allCloudCounts, "查询成功");
            } else {
                return new GeneralResult<>(false, "查询失败，暂无云平台信息");
            }
        } catch (Exception e) {
            log.error("get allCloud count error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getVolumeCount(Long vendorId, Map<Long, String> vendorCache) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult GeneralResult = imsInternalService.getVolume(vendorId);
            if (GeneralResult.isFailed()) {
                result.put("title", "云硬盘数量分布");
                result.put("data", data);
                return new GeneralResult<>(true, result, "查询成功");
            }
            if (GeneralResult.getData() != null) {
                List<Volume> volumes =
                        JSONArray.parseArray(JSONObject.toJSONString(GeneralResult.getData()), Volume.class);
                Map<Long, List<Volume>> volumeMap = volumes.stream().filter(item-> item.getVendorId() !=null).collect(groupingBy(Volume::getVendorId));
                for (Map.Entry<Long, List<Volume>> entry : volumeMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", vendorCache.get(entry.getKey()));
                    jsonObject.put("value", entry.getValue().size());
                    jsonObject.put("type", entry.getValue().get(0).getType());
                    data.add(jsonObject);
                }
            }
            result.put("title", "云硬盘");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get volume count error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getIpPoolCount(Long vendorId, Map<Long, String> vendorCache) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult ipPoolList = imsInternalService.getIpPoolList(null);
            if (ipPoolList.getData() != null) {
                List<IpPool> ipPools = resultToList(ipPoolList, IpPool.class);
                Map<Long, List<IpPool>> dataStoreMap = ipPools.stream().filter(ipPool -> ipPool.getVendorId() != null)
                        .collect(groupingBy(IpPool::getVendorId));
                for (Map.Entry<Long, List<IpPool>> entry : dataStoreMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", vendorCache.get(entry.getKey()));
                    jsonObject.put("value", entry.getValue().size());
                    data.add(jsonObject);
                }
            }
            result.put("title", "IP池");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get ip pool count error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getDataStoreCount(Long vendorId, Long vdcId, Map<Long, String> vendorCache) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult dataStoreResult = imsInternalService.getDataStore(vendorId, vdcId);
            log.info("获取数据存储 : {}", JSONObject.toJSONString(dataStoreResult));
            if (dataStoreResult.getData() != null) {
                List<DataStore> dataStores = resultToList(dataStoreResult, DataStore.class);
                Map<Long, List<DataStore>> dataStoreMap =
                        dataStores.stream().collect(groupingBy(DataStore::getVendorId));
                for (Map.Entry<Long, List<DataStore>> entry : dataStoreMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", vendorCache.get(entry.getKey()));
                    jsonObject.put("value", entry.getValue().size());
                    jsonObject.put("type", entry.getValue().get(0).getType());
                    data.add(jsonObject);
                }
            }
            result.put("title", "数据存储分布");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get data store count error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getVendorCount(Long vendorId, Long vdcId) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            List<CloudVendor> vendorsResult = imsInternalService.getVendorList();
            //获取平台数
            Map<String, List<CloudVendor>> cloudVendors = vendorsResult.stream().filter(cloudVendor -> {
                boolean flag = true;
                if (vendorId != null && vendorId != 0L) {
                    flag = vendorId.equals(cloudVendor.getId());
                }
                if (vdcId != null && vdcId != 0L) {
                    flag = flag && vdcId.equals(cloudVendor.getDcId());
                }
                return flag;
            }).collect(groupingBy(CloudVendor::getType));
            for (Map.Entry<String, List<CloudVendor>> vendorEntry : cloudVendors.entrySet()) {
                JSONObject vendorObject = new JSONObject();
                vendorObject.put("name", vendorEntry.getKey());
                vendorObject.put("value", vendorEntry.getValue().size());
                vendorObject.put("type", vendorEntry.getKey());
                data.add(vendorObject);
            }
            result.put("title", "云平台数量分布");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get vendor count error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<JSONObject> resourceStatus(Long vendorId, Long vdcId, Long clusterId, String condition) {
        if ("vendorStatus".equals(condition)) {
            return getVendorStatus(vendorId, vdcId);
        }
        if ("hostStatus".equals(condition)) {
            return getHostStatus(vendorId, vdcId, clusterId);
        }
        if ("vmsStatus".equals(condition)) {
            return getVmsStatus(vendorId, vdcId, clusterId);
        }
        if ("dataStoreStatus".equals(condition)) {
            return getDataStoreStatus(vendorId, vdcId);
        }
        return new GeneralResult<>(false, "不支持的查询类型");
    }

    private GeneralResult<JSONObject> getHostStatus(Long vendorId, Long vdcId, Long clusterId) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult hostResult = imsInternalService.getHostList(vendorId, vdcId, clusterId, null);
            if (hostResult.getData() != null) {
                List<Server> hostModels =
                        JSONArray.parseArray(JSONObject.toJSONString(hostResult.getData()), Server.class);
                Map<String, List<Server>> hostMap = hostModels.stream().collect(groupingBy(Server::getStatus));
                HashMap<String, List<Server>> resultMap = new HashMap<>();
                for (Map.Entry<String, List<Server>> entry : hostMap.entrySet()) {
                    String key;
                    if (entry.getKey().equals(BocloudStatus.Server.RUNNING.name()) || entry.getKey()
                            .equals(BocloudStatus.Basic.ENABLED.name())) {
                        key = "开机";
                    } else if (entry.getKey().equals(BocloudStatus.Server.STOPPED.name())) {
                        key = "关机";
                    } else if (entry.getKey().equals(BocloudStatus.Server.ERROR.name())) {
                        key = "断开";
                    } else if (entry.getKey().equals(BocloudStatus.Server.EXCEPTION.name())) {
                        key = "异常";
                    } else {
                        key = "其他";
                    }
                    List<Server> serverList = Optional.ofNullable(entry.getValue()).orElse(new ArrayList<>());
                    if (resultMap.containsKey(key)) {
                        List<Server> list = resultMap.get(key);
                        list.addAll(serverList);
                        resultMap.put(key, list);
                    } else {
                        resultMap.put(key, serverList);
                    }
                }
                for (Map.Entry<String, List<Server>> entry : resultMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", entry.getKey());
                    jsonObject.put("value", entry.getValue().size());
                    data.add(jsonObject);
                }
            }
            result.put("title", "主机状态");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get host status error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getVmsStatus(Long vendorId, Long vdcId, Long clusterId) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult vmsResult = imsInternalService.getVmList(vendorId, vdcId, clusterId);
            if (vmsResult.getData() != null) {
                List<CloudServer> serverModels =
                        JSONArray.parseArray(JSONObject.toJSONString(vmsResult.getData()), CloudServer.class);
                Map<String, List<CloudServer>> serverMap =
                        serverModels.stream().collect(groupingBy(CloudServer::getStatus));
                HashMap<String, List<CloudServer>> resultMap = new HashMap<>();
                for (Map.Entry<String, List<CloudServer>> entry : serverMap.entrySet()) {
                    String key;
                    if (entry.getKey().equals(BocloudStatus.Server.RUNNING.name()) || entry.getKey()
                            .equals(BocloudStatus.Basic.ENABLED.name())) {
                        key = "开机";
                    } else if (entry.getKey().equals(BocloudStatus.Server.STOPPED.name())) {
                        key = "关机";
                    } else if (entry.getKey().equals(BocloudStatus.Server.SUSPENDED.name())) {
                        key = "挂起";
                    } else if (entry.getKey().equals(BocloudStatus.Server.EXCEPTION.name())) {
                        key = "异常";
                    } else {
                        key = "其他";
                    }
                    List<CloudServer> serverList = Optional.ofNullable(entry.getValue()).orElse(new ArrayList<>());
                    if (resultMap.containsKey(key)) {
                        List<CloudServer> list = resultMap.get(key);
                        list.addAll(serverList);
                        resultMap.put(key, list);
                    } else {
                        resultMap.put(key, serverList);
                    }
                }
                for (Map.Entry<String, List<CloudServer>> entry : resultMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", entry.getKey());
                    jsonObject.put("value", entry.getValue().size());
                    data.add(jsonObject);
                }
            }
            result.put("title", "云主机状态");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get vms status error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getDataStoreStatus(Long vendorId, Long vdcId) {
        try {
            JSONObject result = new JSONObject();
            JSONArray data = new JSONArray();
            GeneralResult dataStoreResult = imsInternalService.getDataStore(vendorId, vdcId);
            if (dataStoreResult.getData() != null) {
                List<DataStore> dataStores = resultToList(dataStoreResult, DataStore.class);
                Map<String, List<DataStore>> hostMap = dataStores.stream().collect(groupingBy(DataStore::getStatus));
                for (Map.Entry<String, List<DataStore>> entry : hostMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("value", entry.getValue().size());
                    String status = entry.getKey();
                    if ("green".equals(status)) {
                        jsonObject.put("name", "连接");
                    } else {
                        jsonObject.put("name", "断开");
                    }
                    data.add(jsonObject);
                }
            }
            result.put("title", "云存储状态");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get data store status error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private GeneralResult<JSONObject> getVendorStatus(Long vendorId, Long vdcId) {
        try {
            JSONObject result = new JSONObject();
            List<CloudVendor> vendorsResult = imsInternalService.getVendorList();
            //获取平台状态
            Map<String, List<CloudVendor>> cloudVendors = vendorsResult.stream().filter(cloudVendor -> {
                boolean flag = true;
                if (vendorId != null && vendorId != 0L) {
                    flag = vendorId.equals(cloudVendor.getId());
                }
                if (vdcId != null && vdcId != 0L) {
                    flag = flag && vdcId.equals(cloudVendor.getDcId());
                }
                return flag;
            }).collect(groupingBy(CloudVendor::getStatus));
            JSONArray data = new JSONArray();
            for (Map.Entry<String, List<CloudVendor>> entry : cloudVendors.entrySet()) {
                JSONObject jsonObject = new JSONObject();
                if (entry.getKey().equals(BocloudStatus.Basic.ABNORMAL.name())) {
                    jsonObject.put("name", "异常");
                } else if (entry.getKey().equals(BocloudStatus.Basic.NORMAL.name())) {
                    jsonObject.put("name", "正常");
                }
                jsonObject.put("value", entry.getValue().size());
                data.add(jsonObject);
            }
            result.put("title", "平台状态");
            result.put("data", data);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("Get cloud vendor status error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult resourceCapacity(Long vendorId, Long vdcId, Long clusterId, String time, String resourceType,
                                          String condition) {
        if ("getCount".equals(condition)) {
            return getResourceCount(vendorId, vdcId, clusterId);
        }
        if ("getTrend".equals(condition)) {
            List<String> timeList = DateUtil.timeToList(new JSONObject(MapTools.simpleMap("time", time)));
            if (timeList.size() <= 1) {
                return new GeneralResult<>(false, "时间区间选择太短，请重新选择");
            }
            return getResourceTrend(vendorId, vdcId, clusterId, timeList, resourceType);
        }
        return new GeneralResult<>(false, "不支持的查询类型");
    }

    private GeneralResult<JSONObject> getResourceTrend(Long vendorId, Long vdcId, Long clusterId, List<String> timeList,
                                                       String resourceType) {
        JSONObject result = new JSONObject();
        ChargeResultTrend used;
        ChargeResultTrend forecast = null;
        try {
            List<ChargeTrendData> values = new ArrayList<>();
            //获取指定条件下的主机列表
            GeneralResult hostResult = imsInternalService.getHostList(vendorId, vdcId, clusterId, null);
            List<Server> hostModels = new ArrayList<>();
            if (hostResult.getData() != null) {
                hostModels = JSONArray.parseArray(JSONObject.toJSONString(hostResult.getData()), Server.class);
            }
            List<Long> serverIds = hostModels.stream().map(Server::getId).collect(Collectors.toList());
            GeneralResult cloudServerResult = imsInternalService.getVmList(vendorId, vdcId, null);
            List<CloudServer> cloudServerList = new ArrayList<>();
            if (cloudServerResult.getData() != null) {
                cloudServerList =
                        JSONArray.parseArray(JSONObject.toJSONString(cloudServerResult.getData()), CloudServer.class);
            }
            //获取指定条件下的云主机列表（大于目标云主机数）
            //获取使用Ip情况
            GeneralResult ipResult = imsInternalService.getIpNumber(null);
            List<Ip> ips = new ArrayList<>();
            if (ipResult.getData() != null) {
                ips = resultToList(ipResult, Ip.class);
            }
            String name = "";
            List<Object> predictData = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            ChargeTrendData trendData;
            List<String> keys = new ArrayList<>(timeList);
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                        cloudServer -> serverIds.contains(cloudServer.getHostId()) && !BocloudStatus.Server.BUILDING.name()
                                .equals(cloudServer.getStatus()) && cloudServer.getGmtCreate()
                                .before(DateUtil.timeDateProcess(time))).collect(Collectors.toList());
                switch (resourceType) {
                    case "meta.cpu":
                        Integer cpuNum =
                                targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                        data.add(cpuNum);
                        predictData.add(cpuNum);
                        name = "使用量(C）";
                        break;
                    case "meta.mem":
                        BigDecimal memNum = targetVms.stream()
                                .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                        data.add(memNum.doubleValue());
                        predictData.add(memNum.doubleValue());
                        name = "使用量(GB）";
                        break;
                    case "meta.disk":
                        Integer diskNum =
                                targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                        data.add(diskNum);
                        predictData.add(diskNum);
                        name = "使用量(GB）";
                        break;
                    case "meta.ip":
                        List<Ip> resultIps =
                                ips.stream().filter(ip -> ip.getGmtCreate().before(DateUtil.timeDateProcess(time)))
                                        .collect(Collectors.toList());
                        Integer ipNum = resultIps.size();
                        data.add(ipNum);
                        predictData.add(ipNum);
                        name = "使用量(个）";
                        break;
                    default:
                }
            }
            //用当前时间key和data新建当前趋势图（原数组用于预测处理）
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            ArrayList<Object> timeData = new ArrayList<>(data);
            values.add(new ChargeTrendData(name, timeData));
            used = new ChargeResultTrend(timeKeys, values);
            //获取所有数据来做预测
            int oldSize = keys.size();
            LocalDateTime startTime = LocalDateTime
                    .parse(keys.get(0).substring(0, 10) + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            LocalDateTime endTime = LocalDateTime
                    .parse(keys.get(1).substring(0, 10) + " 23:59:59", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            Duration between = Duration.between(startTime, endTime);
            long hours = between.toHours();
            for (int i = 0; i <= oldSize; i++) {
                LocalDateTime lastTime = LocalDateTime.parse(DateUtil.timeProcess(keys.get(keys.size() - 1)),
                        DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                keys.add(DateUtil.localDateTimeFormat(lastTime.plusHours(hours)));
            }
            if (oldSize < 15) {
                // 取样点不足, 使用加权平均算法预测
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    data.add(PredictUtils.weighted(data).intValue());
                }
            } else {
                Integer perdictValue = -1;
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    // arima算法预测, 预测的数值可能小于0; 如果小于0, 则重新预测, 重试5次
                    int retryMax = 5;
                    for (int retry = 0; retry < retryMax; retry++) {
                        perdictValue = PredictUtils.arima(data);
                        if (perdictValue >= 0) {
                            data.add(perdictValue);
                            break;
                        }
                    }
                    // 重试5次失败后, 使用加权平均算法预测
                    if (perdictValue < 0) {
                        data.add(PredictUtils.weighted(data).intValue());
                    }
                }
                if (predictData.size() != 0) {
                    predictData = data.subList(ListUtils.indexOf(data, predictData), data.size());
                }
                predictData = predictData.subList((predictData.size()) / 2, predictData.size());
                trendData = new ChargeTrendData("预测" + name, predictData);
                List<ChargeTrendData> forecastValue = Lists.newArrayList();
                forecastValue.add(trendData);
                List<String> forecastKeys = keys.subList((keys.size()) / 2, keys.size());
                forecast = new ChargeResultTrend(forecastKeys, forecastValue);
            }
            result.put("used", used);
            result.put("forecast", forecast);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get resource trend error", e);
            return new GeneralResult<>(false, null, "查询失败");
        }
    }

    private GeneralResult<JSONArray> getResourceCount(Long vendorId, Long vdcId, Long clusterId) {
        try {
            GeneralResult hostList = imsInternalService.getHostList(vendorId, vdcId, clusterId, null);
            List<Server> servers = new ArrayList<>();
            String cpuUnit = "C";
            if (hostList.getData() != null) {
                servers = JSONArray.parseArray(JSONObject.toJSONString(hostList.getData()), Server.class);
                CloudVendor vendorDetail = imsInternalService.getVendorDetail(vendorId);
                if (null != vendorDetail && CloudProvider.VMWARE.name().equals(vendorDetail.getType())) {
                    cpuUnit = "Mhz";
                }
            }
            JSONObject resourceMap = new JSONObject();
            for (Server server : servers) {
                ServerConfig serverConfig = server.getConfig();
                Double memoryCapacity = Optional.ofNullable(serverConfig.getMemoryCapacity()).orElse(0d);
                if (resourceMap.containsKey("memCapacity")) {
                    Double newMemCapacity = resourceMap.getDouble("memCapacity") + memoryCapacity;
                    resourceMap.put("memCapacity", newMemCapacity);
                } else {
                    resourceMap.put("memCapacity", memoryCapacity);
                }
                Double memUsed = Optional.ofNullable(serverConfig.getMemUsed()).orElse(0d);
                if (resourceMap.containsKey("memUsed")) {
                    Double newMemUsed = resourceMap.getDouble("memUsed") + memUsed;
                    resourceMap.put("memUsed", newMemUsed);
                } else {
                    resourceMap.put("memUsed", memUsed);
                }
                Double diskCapacity = Optional.ofNullable(serverConfig.getDiskCapacity()).orElse(0d);
                if (resourceMap.containsKey("diskCapacity")) {
                    Double newDiskCapacity = resourceMap.getDouble("diskCapacity") + diskCapacity;
                    resourceMap.put("diskCapacity", newDiskCapacity);
                } else {
                    resourceMap.put("diskCapacity", diskCapacity);
                }
                Double diskUsed = Optional.ofNullable(serverConfig.getDiskUsed()).orElse(0d);
                if (resourceMap.containsKey("diskUsed")) {
                    Double newDiskUsed = resourceMap.getDouble("diskUsed") + diskUsed;
                    resourceMap.put("diskUsed", newDiskUsed);
                } else {
                    resourceMap.put("diskUsed", diskUsed);
                }
                Long cpuHz = Optional.ofNullable(serverConfig.getCpuHz()).orElse(0L);
                if (resourceMap.containsKey("cpuHz")) {
                    Long newCpuHz = resourceMap.getLong("cpuHz") + cpuHz;
                    resourceMap.put("cpuHz", newCpuHz);
                } else {
                    resourceMap.put("cpuHz", cpuHz);
                }
                Long cpuUsed = Optional.ofNullable(serverConfig.getCpuUsed()).orElse(0L);
                if (resourceMap.containsKey("cpuUsed")) {
                    Long newCpuUsed = resourceMap.getLong("cpuUsed") + cpuUsed;
                    resourceMap.put("cpuUsed", newCpuUsed);
                } else {
                    resourceMap.put("cpuUsed", cpuUsed);
                }
            }
            JSONArray result = new JSONArray();
            JSONObject cpu = new JSONObject();
            long cpuTotal = Optional.ofNullable(resourceMap.getLong("cpuHz")).orElse(0L);
            long cpuUsed = Optional.ofNullable(resourceMap.getLong("cpuUsed")).orElse(0L);
            long cpuFree = cpuTotal - cpuUsed > 0 ? cpuTotal - cpuUsed : 0;
            double cpuRate = cpuTotal != 0 ? 1.0d * cpuUsed / cpuTotal : 0L;
            cpu.put("name", "CPU使用率");
            cpu.put("unit", cpuUnit);
            cpu.put("total", MathUtils.round(cpuTotal, 2));
            cpu.put("used", MathUtils.round(cpuUsed, 2));
            cpu.put("free", MathUtils.round(cpuFree, 2));
            cpu.put("value", MathUtils.round(cpuRate, 2));
            result.add(cpu);
            JSONObject mem = new JSONObject();
            double memTotal = Optional.ofNullable(resourceMap.getDouble("memCapacity")).orElse(0d);
            double memUsed = Optional.ofNullable(resourceMap.getDouble("memUsed")).orElse(0d);
            double memFree = memTotal - memUsed;
            double memRate = memTotal != 0d ? memUsed / memTotal : 0d;
            mem.put("name", "内存使用率");
            mem.put("unit", "GB");
            mem.put("total", MathUtils.round(memTotal, 2));
            mem.put("used", MathUtils.round(memUsed, 2));
            mem.put("free", MathUtils.round(memFree, 2));
            mem.put("value", MathUtils.round(memRate, 2));
            result.add(mem);
            JSONObject disk = new JSONObject();
            double diskTotal = Optional.ofNullable(resourceMap.getDouble("diskCapacity")).orElse(0d);
            double diskUsed = Optional.ofNullable(resourceMap.getDouble("diskUsed")).orElse(0d);
            double diskFree = diskTotal - diskUsed;
            double diskRate = diskTotal != 0d ? diskUsed / diskTotal : 0d;
            disk.put("name", "存储使用率");
            disk.put("unit", "GB");
            disk.put("total", MathUtils.round(diskTotal, 2));
            disk.put("used", MathUtils.round(diskUsed, 2));
            disk.put("free", MathUtils.round(diskFree, 2));
            disk.put("value", MathUtils.round(diskRate, 2));
            result.add(disk);
            JSONObject ip = new JSONObject();
            //获取IP池中ip数量
            GeneralResult ipList = imsInternalService.getFloatIp(vendorId);
            List<FloatingIp> floatingIps = resultToList(ipList, FloatingIp.class);
            Map<String, List<FloatingIp>> ipMap = floatingIps.stream().collect(groupingBy(FloatingIp::getStatus));
            int totalIpNum = floatingIps.size();
            int runningNum = Optional.ofNullable(ipMap.get("RUNNING")).orElse(new ArrayList<>()).size();
            int activeNum = Optional.ofNullable(ipMap.get("ACTIVE")).orElse(new ArrayList<>()).size();
            int ipUsed = runningNum + activeNum;
            ip.put("name", "IP使用率");
            ip.put("unit", "个");
            ip.put("total", totalIpNum);
            ip.put("used", ipUsed);
            ip.put("free", totalIpNum - ipUsed);
            double rate = totalIpNum == 0 ? 0d : ipUsed * 1d / totalIpNum;
            ip.put("value", MathUtils.round(rate, 2));
            result.add(ip);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get resource count error", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private <T> List<T> resultToList(GeneralResult result, Class<T> clazz) {
        Assert.notNull(result.getData(), "获取列表数据失败");
        JSONObject vendorPager = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
        JSONArray rows = vendorPager.getJSONArray("rows");
        return JSONArray.parseArray(rows.toJSONString(), clazz);
    }

    @Override
    public GeneralResult resourceCapacityNew(Long vendorId, Long vdcId, Long clusterId, String time, String startTime,
                                             String endTime, String resourceType, String condition) {
        Map<String, Object> params = MapTools.simpleMap("time", time);
        params.put("startTime", startTime);
        params.put("endTime", endTime);
        List<String> timeList = DateUtil.timeToList(new JSONObject(params));
        if (timeList.size() <= 1) {
            return new GeneralResult<>(false, "时间区间选择太短，请重新选择");
        }
        if ("getCapacity".equals(condition)) {
            return getCapacityTrendNew(vendorId, vdcId, clusterId, timeList, resourceType);
        }
        if ("getVm".equals(condition)) {
            return getVmTrendNew(vendorId, vdcId, clusterId, timeList);
        }
        return new GeneralResult<>(false, "不支持的查询类型");
    }

    /**
     * 云主机使用趋势
     *
     * @param vendorId
     * @param vdcId
     * @param clusterId
     * @param timeList
     * @return
     */
    private GeneralResult getVmTrendNew(Long vendorId, Long vdcId, Long clusterId, List<String> timeList) {
        try {
            JSONObject result = new JSONObject();
            //使用
            ChargeResultTrend used;
            //预测
            ChargeResultTrend forecast = null;

            List<ChargeTrendData> values = new ArrayList<>();
            //获取指定条件下的主机列表
            GeneralResult hostResult = imsInternalService.getHostList(vendorId, vdcId, clusterId, null);
            List<Server> hostModels = new ArrayList<>();
            if (hostResult.getData() != null) {
                hostModels = JSONArray.parseArray(JSONObject.toJSONString(hostResult.getData()), Server.class);
            }
            //主机id集合
            List<Long> serverIds = hostModels.stream().map(Server::getId).collect(Collectors.toList());
            //根据平台和数据中心取云主机
            GeneralResult cloudServerResult = imsInternalService.getVmList(vendorId, vdcId, null);
            List<CloudServer> cloudServerList = new ArrayList<>();
            if (cloudServerResult.getData() != null) {
                cloudServerList =
                        JSONArray.parseArray(JSONObject.toJSONString(cloudServerResult.getData()), CloudServer.class);
            }
            //获取指定条件下的云主机列表（大于目标云主机数）
            String vmname = "云主机当前使用量";
            List<Object> predictData = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            ChargeTrendData trendData;
            List<String> keys = new ArrayList<>(timeList);
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                        cloudServer -> serverIds.contains(cloudServer.getHostId()) && !BocloudStatus.Server.BUILDING.name()
                                .equals(cloudServer.getStatus()) && cloudServer.getGmtCreate()
                                .before(DateUtil.timeDateProcess(time))).collect(Collectors.toList());
                data.add(targetVms.size());
                predictData.add(targetVms.size());
            }
            //用当前时间key和data新建当前趋势图（原数组用于预测处理）
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            ArrayList<Object> timeData = new ArrayList<>(data);
            values.add(new ChargeTrendData(vmname, timeData));
            used = new ChargeResultTrend(timeKeys, values);
            //获取所有数据来做预测
            int oldSize = keys.size();
            LocalDateTime startTime = LocalDateTime
                    .parse(keys.get(0).substring(0, 10) + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            LocalDateTime endTime = LocalDateTime
                    .parse(keys.get(1).substring(0, 10) + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            Duration between = Duration.between(startTime, endTime);
            long hours = between.toHours();
            for (int i = 0; i <= oldSize; i++) {
                LocalDateTime lastTime = LocalDateTime.parse(DateUtil.timeProcess(keys.get(keys.size() - 1)),
                        DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                keys.add(DateUtil.localDateTimeFormat(lastTime.plusHours(hours)));
            }
            if (oldSize < 15) {
                // 取样点不足, 使用加权平均算法预测
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    data.add(PredictUtils.weighted(data));
                }
            } else {
                Integer perdictValue = -1;
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    // arima算法预测, 预测的数值可能小于0; 如果小于0, 则重新预测, 重试5次
                    int retryMax = 5;
                    for (int retry = 0; retry < retryMax; retry++) {
                        perdictValue = PredictUtils.arima(data);
                        if (perdictValue >= 0) {
                            data.add(perdictValue);
                            break;
                        }
                    }
                    // 重试5次失败后, 使用加权平均算法预测
                    if (perdictValue < 0) {
                        data.add(PredictUtils.weighted(data));
                    }
                }
                if (predictData.size() != 0) {
                    predictData = data.subList(ListUtils.indexOf(data, predictData), data.size());
                }
                predictData = predictData.subList((predictData.size()) / 2, predictData.size());
                trendData = new ChargeTrendData("云主机预测使用量", predictData);
                List<ChargeTrendData> forecastValue = Lists.newArrayList();
                forecastValue.add(trendData);
                List<String> forecastKeys = keys.subList((keys.size()) / 2, keys.size());
                forecast = new ChargeResultTrend(forecastKeys, forecastValue);
            }
            result.put("used", used);
            result.put("forecast", forecast);
            return new GeneralResult<>(true, result, "云主机使用趋势分析查询成功");
        } catch (Exception e) {
            log.error("获取云主机使用趋势失败：{}", e);
            return new GeneralResult<>(false, "云主机使用趋势分析查询失败，原因：" + e.getMessage());
        }
    }

    /**
     * 容量使用趋势
     *
     * @param vendorId
     * @param vdcId
     * @param clusterId
     * @param timeList
     * @param resourceType
     * @return
     */
    private GeneralResult getCapacityTrendNew(Long vendorId, Long vdcId, Long clusterId, List<String> timeList, String resourceType) {
        try {
            JSONObject result = new JSONObject();
            ChargeResultTrend used;
            ChargeResultTrend forecast = null;

            List<ChargeTrendData> values = new ArrayList<>();
            //获取指定条件下的主机列表
            GeneralResult hostResult = imsInternalService.getHostList(vendorId, vdcId, clusterId, null);
            List<Server> hostModels = new ArrayList<>();
            if (hostResult.getData() != null) {
                hostModels = JSONArray.parseArray(JSONObject.toJSONString(hostResult.getData()), Server.class);
            }
            List<Long> serverIds = hostModels.stream().map(Server::getId).collect(Collectors.toList());
            GeneralResult cloudServerResult = imsInternalService.getVmList(vendorId, vdcId, null);
            List<CloudServer> cloudServerList = new ArrayList<>();
            if (cloudServerResult.getData() != null) {
                cloudServerList =
                        JSONArray.parseArray(JSONObject.toJSONString(cloudServerResult.getData()), CloudServer.class);
            }
            //获取指定条件下的云主机列表（大于目标云主机数）
            //获取使用Ip情况
            GeneralResult ipResult = imsInternalService.getIpNumber(null);
            List<Ip> ips = new ArrayList<>();
            if (ipResult.getData() != null) {
                ips = resultToList(ipResult, Ip.class);
            }
            String name = "";
            List<Object> predictData = new ArrayList<>();
            List<Object> data = new ArrayList<>();
            ChargeTrendData trendData;
            List<String> keys = new ArrayList<>(timeList);
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                        cloudServer -> serverIds.contains(cloudServer.getHostId()) && !BocloudStatus.Server.BUILDING.name()
                                .equals(cloudServer.getStatus()) && cloudServer.getGmtCreate()
                                .before(DateUtil.timeDateProcess(time))).collect(Collectors.toList());
                switch (resourceType) {
                    case "meta.cpu":
                        Integer cpuNum =
                                targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                        data.add(cpuNum);
                        predictData.add(cpuNum);
                        name = "使用量(C）";
                        break;
                    case "meta.mem":
                        BigDecimal memNum = targetVms.stream()
                                .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                        data.add(memNum.doubleValue());
                        predictData.add(memNum.doubleValue());
                        name = "使用量(GB）";
                        break;
                    case "meta.disk":
                        Integer diskNum =
                                targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                        data.add(diskNum);
                        predictData.add(diskNum);
                        name = "使用量(GB）";
                        break;
                    case "meta.ip":
                        List<Ip> resultIps =
                                ips.stream().filter(ip -> ip.getGmtCreate().before(DateUtil.timeDateProcess(time)))
                                        .collect(Collectors.toList());
                        Integer ipNum = resultIps.size();
                        data.add(ipNum);
                        predictData.add(ipNum);
                        name = "使用量(个）";
                        break;
                    default:
                }
            }
            //用当前时间key和data新建当前趋势图（原数组用于预测处理）
            ArrayList<String> timeKeys = new ArrayList<>(keys);
            ArrayList<Object> timeData = new ArrayList<>(data);
            values.add(new ChargeTrendData(name, timeData));
            used = new ChargeResultTrend(timeKeys, values);
            //获取所有数据来做预测
            int oldSize = keys.size();
            LocalDateTime startTime = LocalDateTime
                    .parse(keys.get(0).substring(0, 10) + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            LocalDateTime endTime = LocalDateTime
                    .parse(keys.get(1).substring(0, 10) + " 23:59:59", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            Duration between = Duration.between(startTime, endTime);
            long hours = between.toHours();
            for (int i = 0; i <= oldSize; i++) {
                LocalDateTime lastTime = LocalDateTime.parse(DateUtil.timeProcess(keys.get(keys.size() - 1)),
                        DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                keys.add(DateUtil.localDateTimeFormat(lastTime.plusHours(hours)));
            }
            if (oldSize < 15) {
                // 取样点不足, 使用加权平均算法预测
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    data.add(PredictUtils.weighted(data).intValue());
                }
            } else {
                Integer perdictValue = -1;
                for (int index = 0; index < keys.size() - oldSize; index++) {
                    // arima算法预测, 预测的数值可能小于0; 如果小于0, 则重新预测, 重试5次
                    int retryMax = 5;
                    for (int retry = 0; retry < retryMax; retry++) {
                        perdictValue = PredictUtils.arima(data);
                        if (perdictValue >= 0) {
                            data.add(perdictValue);
                            break;
                        }
                    }
                    // 重试5次失败后, 使用加权平均算法预测
                    if (perdictValue < 0) {
                        data.add(PredictUtils.weighted(data).intValue());
                    }
                }
                if (predictData.size() != 0) {
                    predictData = data.subList(ListUtils.indexOf(data, predictData), data.size());
                }
                predictData = predictData.subList((predictData.size()) / 2, predictData.size());
                trendData = new ChargeTrendData("预测" + name, predictData);
                List<ChargeTrendData> forecastValue = Lists.newArrayList();
                forecastValue.add(trendData);
                List<String> forecastKeys = keys.subList((keys.size()) / 2, keys.size());
                forecast = new ChargeResultTrend(forecastKeys, forecastValue);
            }
            result.put("used", used);
            result.put("forecast", forecast);
            return new GeneralResult<>(true, result, "容量使用趋势分析查询成功");
        } catch (Exception e) {
            return new GeneralResult<>(false, "容量使用趋势分析查询失败，原因：" + e.getMessage());
        }

    }

    @Override
    public GeneralResult cloudCapacity(Pager pager, String vendorName) {
        try {
            int page = pager.getPage();
            int rows = pager.getRows();
            List<CloudVendor> cloudVendorList = imsInternalService.getVendorList();
            cloudVendorList = cloudVendorList.stream().filter(
                    cloudVendor -> (!cloudVendor.getIsPublic())).collect(Collectors.toList());
            if (ListTool.isEmpty(cloudVendorList)) {
                return new GeneralResult<>(false, "暂无私有云平台物理、虚拟资源总量信息");
            }
            //总量
            Double hostMem = 0D;
            Double hostDisk = 0D;
            Long hostCpu = 0L;
            Integer hostNum = 0;
            Double vmMem = 0D;
            Double vmDisk = 0D;
            Long vmCpu = 0L;
            Integer vmNum = 0;
            CloudResourceCapacity cloudResourceCapacity = new CloudResourceCapacity();
            List<CloudResourceCapacity> cloudResourceCapacities = new ArrayList<>();
            for (CloudVendor cloudVendor : cloudVendorList) {
                CloudResourceCapacity cloudVendorCapacity = new CloudResourceCapacity();
                GeneralResult hostList = imsInternalService.getHostList(cloudVendor.getId(), null, null, null);
                List<Server> servers = new ArrayList<>();
                if (hostList.getData() != null && hostList.isSuccess()) {
                    servers = JSONArray.parseArray(JSONObject.toJSONString(hostList.getData()), Server.class);
                    servers = servers.stream().filter(server -> (server.getVendorId().equals(cloudVendor.getId())))
                            .collect(Collectors.toList());
                }
                if (ListTool.isEmpty(servers)) {
                    continue;
                }
                hostNum += servers.size();
                Double mem = 0D;
                Double disk = 0D;
                Long cpu = 0L;
                for (Server server : servers) {
                    ServerConfig serverConfig = server.getConfig();
                    if (null != serverConfig) {
                        disk += Optional.ofNullable(serverConfig.getDiskCapacity()).orElse(0d);

                        //从平台获取超分比
                        String str = StringEscapeUtils.unescapeJava(cloudVendor.getAuthentication());
                        JSONObject authentication = JSONObject.parseObject(str);
                        // 超分比
                        Long excessRatio = 0L;
                        if (null != authentication.get("excessRatio") && CloudProvider.VMWARE.name().equals(cloudVendor.getType())) {
                            excessRatio = Long.valueOf(String.valueOf(authentication.get("excessRatio")));
                        }
                        Integer cpuCore = Optional.ofNullable(serverConfig.getCpuSum()).orElse(0);
                        Double memCapacity = Optional.ofNullable(serverConfig.getMemoryCapacity()).orElse(0d);
                        if (null != excessRatio && 0L != excessRatio) {
                            cpu += cpuCore * excessRatio;
                            mem += memCapacity.longValue() * excessRatio;
                        } else {
                            cpu += (long) cpuCore;
                            mem += memCapacity.longValue();
                        }
                    }
                }
                hostCpu += cpu;
                hostDisk += disk;
                hostMem += mem;
                cloudVendorCapacity.setHostCpu(cpu);
                cloudVendorCapacity.setHostMem(MathUtils.round(mem, 2));
                cloudVendorCapacity.setHostDisk(MathUtils.round(disk / 1024D, 2));
                cloudVendorCapacity.setHostNum(servers.size());
                cloudVendorCapacity.setVendorName(cloudVendor.getName());
                cloudVendorCapacity.setVendorType(cloudVendor.getType());
                //云平台虚拟资源总量
                GeneralResult vmsRes = imsInternalService.selectAllByVendorId(cloudVendor.getId(), cloudVendor.getType());
                if (vmsRes.isSuccess() && null != vmsRes.getData()) {
                    List<Map<String, Object>> data = (List<Map<String, Object>>) vmsRes.getData();
                    for (Map<String, Object> data1 : data) {
                        for (Map.Entry<String, Object> entry : data1.entrySet()) {
                            if ("disk".equals(entry.getKey()) || "DISK".equals(entry.getKey())) {
                                if (null != entry.getValue()) {
                                    Double diskvm =
                                            MathUtils.round(Double.parseDouble(entry.getValue().toString()) / 1024D, 2);
                                    cloudVendorCapacity.setVmDisk(diskvm);
                                    vmDisk += diskvm;
                                }
                            } else if ("mem".equals(entry.getKey()) || "MEM".equals(entry.getKey())) {
                                if (null != entry.getValue()) {
                                    Double memVm = MathUtils.round(Double.parseDouble(entry.getValue().toString()), 2);
                                    cloudVendorCapacity.setVmMem(memVm);
                                    vmMem += memVm;
                                }
                            } else if ("cpu".equals(entry.getKey()) || "CPU".equals(entry.getKey())) {
                                if (null != entry.getValue()) {
                                    Long cpuVm = Long.valueOf(String.valueOf(entry.getValue()));
                                    cloudVendorCapacity.setVmCpu(cpuVm);
                                    vmCpu += cpuVm;
                                }
                            } else if ("host".equals(entry.getKey()) || "HOST".equals(entry.getKey())) {
                                if (null != entry.getValue()) {
                                    Integer hostVm = Integer.parseInt(String.valueOf(entry.getValue()));
                                    cloudVendorCapacity.setVmNum(hostVm);
                                    vmNum += hostVm;
                                }
                            }
                        }
                    }
                }
                cloudResourceCapacities.add(cloudVendorCapacity);
            }
            cloudResourceCapacity.setHostNum(hostNum);
            cloudResourceCapacity.setHostCpu(hostCpu);
            cloudResourceCapacity.setHostDisk(MathUtils.round(hostDisk / 1024D, 2));
            cloudResourceCapacity.setHostMem(MathUtils.round(hostMem, 2));
            cloudResourceCapacity.setVmNum(vmNum);
            cloudResourceCapacity.setVmCpu(vmCpu);
            cloudResourceCapacity.setVmMem(MathUtils.round(vmMem, 2));
            cloudResourceCapacity.setVmDisk(MathUtils.round(vmDisk, 2));
            if (null != vendorName) {
                cloudResourceCapacities = cloudResourceCapacities.stream()
                        .filter(cloudResourceCapacity1 -> (cloudResourceCapacity1.getVendorName().contains(vendorName)))
                        .collect(Collectors.toList());
            }
            GridBean gridBean = GridHelper.getBean(page, rows, cloudResourceCapacities.size(), cloudResourceCapacities);
            cloudResourceCapacity.setCloudResourceCapacities(gridBean);

            return new GeneralResult(true, cloudResourceCapacity, "私有云平台物理、虚拟资源总量查询成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new GeneralResult(false, "私有云平台物理、虚拟资源总量查询失败：" + e.getMessage());
        }

    }
}
