package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessCollectionObject;
import com.bocloud.cos.entity.assess.AssessCollectionObjectDetail;
import com.bocloud.cos.repository.assess.AssessCollectionObjectDetailRepository;
import com.bocloud.cos.repository.assess.AssessCollectionObjectRepository;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 考核采集对象业务逻辑处理
 *
 * @author lym
 */
@Service
public class AssessCollectionObjectService {

    @Autowired
    private AssessCollectionObjectRepository collectionObjectRepository;

    @Autowired
    private AssessCollectionObjectDetailRepository collectionObjectDetailRepository;

    public GridBean<AssessCollectionObject> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        int total = collectionObjectRepository.count(pager.getParams());
        List<AssessCollectionObject> list = collectionObjectRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        list.forEach(collectionObject -> {
            List<AssessCollectionObjectDetail> assessCollectionObjectDetails = collectionObjectDetailRepository.listByObjId(collectionObject.getId());
            collectionObject.setResourceList(assessCollectionObjectDetails);
        });
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    @Transactional(rollbackFor = Exception.class)
    public void save(AssessCollectionObject collectionObject, RequestContext context) {
        Assert.notEmpty(collectionObject.getResourceList(), "请选择资源");
        saveOf(collectionObject, context);
        collectionObjectRepository.save(collectionObject);

        batchSaveObjectDetail(collectionObject.getId(), collectionObject.getResourceList());
    }

    public void update(AssessCollectionObject collectionObject, RequestContext context) {
        AssessCollectionObject ac = collectionObjectRepository.get(AssessCollectionObject.class, collectionObject.getId());
        Assert.notNull(ac, "考核采集对象不存在");
        ac.setName(collectionObject.getName());
        ac.setCode(collectionObject.getCode());
        ac.setRemark(collectionObject.getRemark());
        updateOf(ac, context);
        boolean update = collectionObjectRepository.update(ac);
        if (update) {
            processObjectDetail(ac.getId(), collectionObject.getResourceList());
        }
    }

    private void processObjectDetail(Long objectId, List<AssessCollectionObjectDetail> resourceList) {

        List<AssessCollectionObjectDetail> oldObjectDetails = collectionObjectDetailRepository.listByObjId(objectId);
        List<String> oldResourceTypes = oldObjectDetails.stream().map(AssessCollectionObjectDetail::getResourceType).toList();
        List<String> newResourceTypes = resourceList.stream().map(AssessCollectionObjectDetail::getResourceType).toList();

        List<String> addObjectDetails = newResourceTypes.stream().filter(x -> !oldResourceTypes.contains(x)).collect(Collectors.toList());
        List<String> delObjectDetails = oldResourceTypes.stream().filter(x -> !newResourceTypes.contains(x)).collect(Collectors.toList());

        if (!addObjectDetails.isEmpty()) {
            resourceList.forEach(resource -> {
                if (addObjectDetails.contains(resource.getResourceType())) {
                    AssessCollectionObjectDetail objectDetail = new AssessCollectionObjectDetail();
                    objectDetail.setCollectionObjId(objectId);
                    objectDetail.setResourceName(resource.getResourceName());
                    objectDetail.setResourceType(resource.getResourceType());
                    collectionObjectDetailRepository.save(objectDetail);
                }
            });
        }

        if (!delObjectDetails.isEmpty()) {
            List<Long> ids = new ArrayList<>();
            oldObjectDetails.forEach(objectDetail -> {
                if (delObjectDetails.contains(objectDetail.getResourceType())) {
                    ids.add(objectDetail.getId());
                }
            });
            collectionObjectDetailRepository.deleteByIds(ids);
        }
    }

    private void batchSaveObjectDetail(Long objectId, List<AssessCollectionObjectDetail> resourceList) {
        List<AssessCollectionObjectDetail> objectDetails = resourceList.stream()
                .map(resource -> {
                    AssessCollectionObjectDetail objectDetail = new AssessCollectionObjectDetail();
                    objectDetail.setCollectionObjId(objectId);
                    objectDetail.setResourceName(resource.getResourceName());
                    objectDetail.setResourceType(resource.getResourceType());
                    return objectDetail;
                })
                .collect(Collectors.toList());

        collectionObjectDetailRepository.batchSave(objectDetails, objectDetails.size());
    }

    @Transactional(rollbackFor = Exception.class)
    public void remove(Long id) {
        collectionObjectDetailRepository.deleteByCollectionObjId(id);
        collectionObjectRepository.deleteById(id);
    }

    private void saveOf(AssessCollectionObject collectionObject, RequestContext context) {
        collectionObject.setCreatorId(context.getTarget());
        collectionObject.setGmtCreate(new Date());
    }

    private void updateOf(AssessCollectionObject collectionObject, RequestContext context) {
        collectionObject.setMenderId(context.getTarget());
        collectionObject.setGmtModify(new Date());
    }
}
