package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessRule;
import com.bocloud.cos.repository.assess.AssessRuleRepository;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 考核规则业务逻辑处理
 *
 * @author lym
 */
@Slf4j
@Service
public class AssessRuleService {

    @Autowired
    private AssessRuleRepository assessRuleRepository;

    public void save(AssessRule assessRule, RequestContext context) {
        saveOf(assessRule, context);
        assessRuleRepository.save(assessRule);
    }

    public GridBean<AssessRule> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = assessRuleRepository.count(pager.getParams());
        List<AssessRule> list = assessRuleRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public void update(AssessRule assessRule, RequestContext context) {
        AssessRule ar = getAssessRule(assessRule.getId());
        ar.setRuleContent(assessRule.getRuleContent());
        ar.setSubmissionItem(assessRule.getSubmissionItem());
        ar.setScoreSetting(assessRule.getScoreSetting());
        updateOf(ar, context);
        assessRuleRepository.update(ar);
    }

    public void remove(Long id, RequestContext context) {
        AssessRule assessRule = getAssessRule(id);
        assessRule.setDeleted(true);
        updateOf(assessRule, context);
        assessRuleRepository.update(assessRule);
    }

    private AssessRule getAssessRule(Long id) {
        AssessRule assessRule = assessRuleRepository.get(AssessRule.class, id);
        Assert.notNull(assessRule, "该考核指标不存在");
        return assessRule;
    }

    private void saveOf(AssessRule assessRule, RequestContext context) {
        assessRule.setCreatorId(context.getTarget());
        assessRule.setGmtCreate(new Date());
    }

    private void updateOf(AssessRule assessRule, RequestContext context) {
        assessRule.setMenderId(context.getTarget());
        assessRule.setGmtModify(new Date());
    }
}
