package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessRule;
import com.bocloud.cos.entity.assess.AssessTask;
import com.bocloud.cos.entity.assess.AssessTaskMetric;
import com.bocloud.cos.repository.assess.AssessRuleRepository;
import com.bocloud.cos.repository.assess.AssessTaskMetricRepository;
import com.bocloud.cos.repository.assess.AssessTaskRepository;
import com.bocloud.cos.enums.AssessTaskStatusEnum;
import com.megatron.common.model.RequestContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AssessTaskMetricService {

    @Autowired
    private AssessTaskMetricRepository taskMetricRepository;

    @Autowired
    private AssessRuleRepository ruleRepository;

    @Autowired
    private AssessTaskRepository taskRepository;

    public void taskBindMetric(Long taskId, List<Long> metricIds, RequestContext context) {
        if (CollectionUtils.isEmpty(metricIds)) {
            throw new IllegalArgumentException("请保证至少存在一项考核指标");
        }

        List<Long> addMetricIds;
        List<Long> delMetricIds = new ArrayList<>();

        List<AssessTaskMetric> assessTaskMetrics = taskMetricRepository.listByTaskId(taskId);
        List<Long> oldMetricIds = assessTaskMetrics.stream().map(AssessTaskMetric::getMetricId).toList();

        if (oldMetricIds.isEmpty()) {
            addMetricIds = metricIds;
        } else {
            addMetricIds = metricIds.stream().filter(x -> !oldMetricIds.contains(x)).collect(Collectors.toList());
            delMetricIds = oldMetricIds.stream().filter(x -> !metricIds.contains(x)).collect(Collectors.toList());
        }

        // 添加新的指标
        if (!addMetricIds.isEmpty()) {
            List<AssessTaskMetric> newAssessTaskMetrics = new ArrayList<>();
            List<AssessRule> assessRules = ruleRepository.queryByMetricId(addMetricIds);
            assessRules.forEach(rule -> {
                AssessTaskMetric assessTaskMetric = new AssessTaskMetric();
                assessTaskMetric.setTaskId(taskId);
                assessTaskMetric.setMetricId(rule.getMetricId());
                assessTaskMetric.setMetricRuleId(rule.getId());
                assessTaskMetric.setMaxScore(rule.getMaxScore());
                assessTaskMetric.setWeight(0);
                assessTaskMetric.setCreatorId(context.getTarget());
                assessTaskMetric.setGmtCreate(new Date());
                newAssessTaskMetrics.add(assessTaskMetric);
            });
            taskMetricRepository.batchSave(newAssessTaskMetrics, newAssessTaskMetrics.size());
        }

        // 删除移除的指标
        if (!delMetricIds.isEmpty()) {
            taskMetricRepository.removeByMetricIds(delMetricIds);
        }

        // 更新任务状态为创建中
        updateTaskStatus(taskId, AssessTaskStatusEnum.CREATEING.getCode(), context);
    }

    private void updateTaskStatus(Long taskId, String status, RequestContext context) {
        AssessTask task = taskRepository.get(AssessTask.class, taskId);
        task.setStatus(status);
        task.setMenderId(context.getTarget());
        task.setGmtModify(new Date());
        taskRepository.update(task);
    }

    public void updateAssessTaskMetric(List<AssessTaskMetric> taskMetrics, RequestContext context) {
        if (CollectionUtils.isEmpty(taskMetrics)) {
            return;
        }
        taskMetrics.forEach(taskMetric -> {
            taskMetric.setMenderId(context.getTarget());
            taskMetric.setGmtModify(new Date());
        });
        taskMetricRepository.batchUpdate(taskMetrics, taskMetrics.size());

        // 更新任务状态为待发布
        updateTaskStatus(taskMetrics.get(0).getTaskId(), AssessTaskStatusEnum.WAIT_PUBLISH.getCode(), context);
    }

    public List<AssessTaskMetric> listDetailByTaskId(Long taskId, String metricName) {
        return taskMetricRepository.listDetailByTaskId(taskId, metricName);
    }
}
