package com.bocloud.cos.service.internal;

import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * CMS监控内部访问
 *
 * @author lym
 */
@Slf4j
@Component
public class CmsInternalService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create("cms");

    public Double prometheus(String expr, String time) {
        try {
            Map<String, Object> paramMap = MapTools.simpleMap("expr", expr);
            paramMap.put("time", time);
            paramMap.put("simple", true);
            RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/prometheus", HttpMethod.GET, headers, paramMap);
            GeneralResult<?> result = remoteService.invoke();
            if (result.isFailed()) {
                return null;
            }
            return Double.valueOf(result.getData().toString());
        } catch (Exception e) {
            log.error("查询监控数据失败：{}", e);
        }
        return 0.00;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
    }
}
