package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.CloudServer;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * cmp服务内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月22日
 */
@Component
public class ImsServerService {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;

    private static Service service = Service.create("cmp");

    public GeneralResult vmsRestore(Long id, String plugin, RequestContext context) {
        Map<String, Object> params = MapTools.simpleMap(Common.ACTION, "restore");
        params.put(Common.ACTION, "restore");
        Map<String, Object> headers = MapTools.simpleMap(Common.API_KEY, context.getApiKey());
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(service, "/plugins/" + plugin + "/v1/vms/" + id, HttpMethod.PATCH, headers, params);
        remoteService.setDataFormat(PostDataFormat.RAW);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult(false, result.getMessage());
        }
        return result;
    }

    public GeneralResult serverDetail(String apiKey, Long serverId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.API_KEY, apiKey);
        JSONObject object = new JSONObject();
        object.put("serverId", serverId);
        object.put(Common.CONDITION, "basic");
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms", HttpMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    /**
     * 分页查询虚拟机列表
     *
     * @param token
     * @param pager
     * @return
     */
    public GeneralResult<GridBean<CloudServer>> list(String token, Pager pager) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.TOKEN, token);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(service, "/v1/vms", HttpMethod.GET, headers, requestMap);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return result;
        }
        String content = JSONObject.toJSONString(result.getData());
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return new GeneralResult<>(true, gridBean, result.getMessage());
    }

}
