package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.Cms;
import com.bocloud.cos.entity.internel.Kvs;
import com.bocloud.cos.entity.internel.Rds;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author tyl
 * @since 2020/10/26 15:46
 */
@Component
public class PmsInternalService implements InitializingBean {
    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create("cmp");

    @Autowired
    public PmsInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    public GeneralResult rdsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/plugins/mms/v1/rds/" + id + "/restore", HttpMethod.PATCH, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult(false, result.getMessage());
        }
        return result;
    }

    public Rds rdsDetail(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/plugins/mms/v1/rds/" + id + "/basics", HttpMethod.GET, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Rds.class);
    }

    public GeneralResult cmsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/plugins/mms/v1/cms/" + id + "/restore", HttpMethod.PATCH, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult(false, result.getMessage());
        }
        return result;
    }

    public Cms cmsDetail(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/plugins/mms/v1/cms/" + id + "/basics", HttpMethod.GET, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Cms.class);
    }

    public GeneralResult kvsRestore(Long id) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/plugins/mms/v1/kvs/" + id + "/restore", HttpMethod.PATCH, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult(false, result.getMessage());
        }
        return result;
    }

    public Kvs kvsDetail(Long id) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/mms/v1/kvs/" + id + "/basics", HttpMethod.GET, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Kvs.class);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
