package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.internel.User;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Component
public class SmsUserService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsUserService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create("sms");

    /**
     * 获取用户详情
     *
     * @param id 用户id
     * @return
     */
    public GeneralResult<User> detail(Long id) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/users/" + id, HttpMethod.GET,
                headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, User.class), "success");
    }

    public List<CloudService> getUserServices(String apiKey) {
        headers.put(Common.API_KEY, apiKey);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/users/roles/services", HttpMethod.GET, headers, null);
        GeneralResult result = remote.invoke();
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudService.class);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

}
